# ----------------------------------- #
#     Functions for cvd_hcUse.Rnw     #
# ----------------------------------- #
# This file includes all functions needed for the marginal structural analysis code investigating the association of
# CVD or metabolic disorders and hormonal contraceptive use.
# Functions included:
# 1)  Forest plot (_alt allows putting the estimate, 95% CI, and P-value table to the side)
# 2)  


diagnosis_function <- function(d, condNme, dxNme, minAgeNme, ageVar){
#   cond <- d$na_accord_diagnosis_label == condNme
   d$cond <- condNme
   d[[dxNme]] <- NA
   split(d[[dxNme]], d$cfar_pid) <- lapply(split(d$cond, d$cfar_pid),
                                       FUN=function(y){
                                         if(any(d$cond)){
                                          tmp <- max(as.numeric(y),na.rm=TRUE)
                                       } else {
                                          tmp <- NA
                                       }
                                       return(tmp)
                                       })

   d[[minAgeNme]] <- NA
   split(d[[minAgeNme]], d$cfar_pid) <- lapply(split(d, d$cfar_pid),
                                         FUN=function(y){
                                            if(!all(y[[dxNme]] == 0)){
                                               tmp <- y[which(y$cond == 1),]
                                               age <- min(tmp[[ageVar]],na.rm=TRUE)
                                            } else {
                                               age <- NA
                                            }
                                            return(age)
                                      })

   dxNme2 <- gsub('_overall','',dxNme)
   indNme <- paste0(dxNme2,'_prior_to_first_visit')
   d[[indNme]] <- as.numeric(!is.na(d[[minAgeNme]]) & d[[minAgeNme]] <= d$age_at_first_visit + 28/365.25)

   d$cond <- NULL

   return(d)
}

baseline_value_function <- function(d,
                                    firstAge='age_at_first_visit',
                                    condNme,
                                    minLim,
                                    maxLim,
                                    baseNme,
                                    baseAgeNme){
   if(!is.null(condNme)){
      dSub <- d[condNme,]
   } else {
      dSub <- d
   }
   dSub$dateDiff <- 365.25*(as.numeric(dSub$age_at_result_date - dSub[[firstAge]]))
   dSub$in_window <- dSub$dateDiff >= minLim & dSub$dateDiff <= maxLim
   dSub_sub <- dSub[dSub$in_window == 1,]
   dSub_sub <- dSub_sub[order(dSub_sub$cfar_pid, abs(dSub_sub$dateDiff)),]
   dSub_sub2 <- dSub_sub[!duplicated(dSub_sub$cfar_pid),]
   dSub_sub2[[baseNme]] <- dSub_sub2$result_numeric
   dSub_sub2[[baseAgeNme]] <- dSub_sub2$age_at_result_date

   d[[baseNme]] <- dSub_sub2[match(d$cfar_pid, dSub_sub2$cfar_pid, nomatch=NA),baseNme]
   d[[baseAgeNme]] <- dSub_sub2[match(d$cfar_pid, dSub_sub2$cfar_pid, nomatch=NA),baseAgeNme]

   return(d)
}

dataPrep <- function(df,
                     eventAge,
                     maxEventAge,
                     event,
                     prevalentEvent){

   # Excluding subjects who had prevalent cases of the outcome of interest (prevalent based on first visit):
   condPrev <- df[[prevalentEvent]] == 1
   d <- df[!condPrev,]

#   # Excluding pregnancy records:
#   d <- d[d$during_pregnancy == 0,]

#   # Creating an age_at_first_record for those who were pregnant at their first visit:
#   d <- d[order(d$cfar_pid,d$age),]
#   d$age_at_first_record <- NA
#   split(d$age_at_first_record, d$cfar_pid) <- lapply(split(d$age, d$cfar_pid), FUN=function(y){
#                                                  y[1] })

   # Now that there is a new first record, must also exclude those who had an event while pregnant if pregnant at first visit:
   condNewPrev <- !is.na(d[[eventAge]]) & d[[eventAge]] <= d$age_at_first_record
   d <- d[!condNewPrev,]

   # Need to reorder month variable:
   d$monthOriginal <- d$month
   d$month <- NA
   split(d$month, d$cfar_pid) <- lapply(split(d$cfar_pid, d$cfar_pid), FUN=function(y){
                                    seq(1,length(y)) })

   # Subsetting records so that those having the event are subsetted at the age of the event or right after a pregnancy if the event occurred
   # during a pregnancy:
   d$maxAge <- NA
   split(d$maxAge, d$cfar_pid) <- lapply(split(d[,c(eventAge,maxEventAge)],d$cfar_pid), FUN=function(y){
                                     if(any(!is.na(y[[eventAge]]) | !is.na(y[[maxEventAge]]))){
                                       tmp <- max(c(y[[eventAge]],y[[maxEventAge]]),na.rm=TRUE)
                                     } else {
                                       tmp <- NA
                                     }
                                     return(tmp) })
   condEvent <- is.na(d[[eventAge]]) |
#                (!is.na(d[[eventAge]]) & d$age <= d[[eventAge]])
                (!is.na(d[[eventAge]]) & d$age <= d$maxAge)
   d <- d[condEvent,]


   # Also want to create overall HC use indicators (not time-varying ones) and cumYrsHCUse***Max for each dataList
   # Will use '_sub' to differentiate from those in the df data frame
   # New HC use indicators overall:
   d$hc_use_any_sub <- NA
   split(d$hc_use_any_sub, d$cfar_pid) <- lapply(split(d$hc_use_any_indicator, d$cfar_pid), FUN=function(y){
                                             max(y,na.rm=TRUE) })
   d$hc_use_any_sub.factor <- factor(d$hc_use_any_sub, levels=c(0,1), labels=c('No HC Use','HC Use'))

   d$hc_use_oral_sub <- NA
   split(d$hc_use_oral_sub, d$cfar_pid) <- lapply(split(d$hc_use_oral_indicator, d$cfar_pid), FUN=function(y){
                                             max(y,na.rm=TRUE) })
   d$hc_use_oral_sub.factor <- factor(d$hc_use_oral_sub, levels=c(0,1), labels=c('No oral HC Use','Oral HC Use'))

   d$hc_use_depo_sub <- NA
   split(d$hc_use_depo_sub, d$cfar_pid) <- lapply(split(d$hc_use_depo_indicator, d$cfar_pid), FUN=function(y){
                                             max(y,na.rm=TRUE) })
   d$hc_use_depo_sub.factor <- factor(d$hc_use_depo_sub, levels=c(0,1), labels=c('No Depo Use','Depo Use'))
   
   # Max cumulative Yrs on HC:
   d$cumYrsHCUseAnyMaxSub <- NA
   split(d$cumYrsHCUseAnyMaxSub, d$cfar_pid) <- lapply(split(d$cumYrsHCUseAny, d$cfar_pid), FUN=function(y){
                                                   max(y, na.rm=TRUE) })

   d$cumYrsHCUseOralMaxSub <- NA
   split(d$cumYrsHCUseOralMaxSub, d$cfar_pid) <- lapply(split(d$cumYrsHCUseOral, d$cfar_pid), FUN=function(y){
                                                   max(y, na.rm=TRUE) })

   d$cumYrsHCUseDepoMaxSub <- NA
   split(d$cumYrsHCUseDepoMaxSub, d$cfar_pid) <- lapply(split(d$cumYrsHCUseDepo, d$cfar_pid), FUN=function(y){
                                                   max(y, na.rm=TRUE) })



   return(d)
}

df_prep_function <- function(summ,
                             varRows,
                             add1=TRUE){
    which_rows <- NULL
    for(i in 1:length(varRows)){
        which_rows <- c(which_rows, grep(varRows[i], rownames(summ), value=FALSE))
    }
    if(add1){
       which_rows <- which_rows + 1
    }

    return(which_rows)

}



createDF <- function(which_rows,
                     covariate,
                     summ1,
                     summ2,
                     summ3,
                     summ2Nmes = NULL,
                     summ3Nmes = NULL,
                     spaceHolder,
                     pHolder,
                     pOrder = c('t_lab_yrs','baseline_weight','site','group','age','year','sex','cd4n','log10'),
                     mod,
                     predNme = 'weight',
                     hrNme = 'Hazard Ratio'){

   if(hrNme %in% c('Odds Ratio','Hazard Ratio')){
      refNme <- '1.00'
   } else {
      refNme <- '0.00'
   }

   # Extracting betas from each of the 3 summaries:
    hr1 <- dfCols_function(which_rows = which_rows,
                           summ = summ1,
                           colName = 'Effect')
    if(!is.null(summ2)){
#       rwnmes2 <- rownames(summ2)[-c(grep('Odds',rownames(summ2)),grep('Hazard',rownames(summ2)))]
       rwnmes2 <- rownames(summ2)[which_rows-1]
#       rwnmes2 <- rwnmes2[which_rows-1]
       hr2 <- dfCols_function(which_rows = which_rows,
                              summ = summ2,
                              colName = 'Effect')
       names(hr2) <- rwnmes2
#       summ2Nmes <- c('t_lab_yrs',predNme,'age','year','cd4n','log10')
       cols2Choose <- grepl(paste(summ2Nmes,collapse='*|'),names(hr2))
       hr2 <- hr2[cols2Choose]
    }

    if(!is.null(summ3)){
#       rwnmes3 <- rownames(summ3)[-c(grep('Odds',rownames(summ3)),grep('Hazard',rownames(summ3)))]
#       rwnmes3 <- rwnmes3[which_rows-1]
       rwnmes3 <- rownames(summ3)[which_rows-1]
       hr3 <- dfCols_function(which_rows = which_rows,
                              summ = summ3,
                              colName = 'Effect')
      names(hr3) <- rwnmes3
#       summ3Nmes <- c('cd4n')
       cols3Choose <- grepl(paste(summ3Nmes,collapse='*|'),names(hr3))
       hr3 <- hr3[cols3Choose]
    }

   # HRs with spaces inserted:
   hrWspace <- vector('character',length(covariate))
   hrWspace[which(spaceHolder == 1)] <- hr1
   if(!is.null(summ2)){
      hrWspace[which(spaceHolder == 2)] <- hr2
   }
   if(!is.null(summ3)){
      hrWspace[which(spaceHolder == 3)] <- hr3
   }
   hrWspace[which(spaceHolder == 'Ref')] <- refNme

   # Extracting LCIs from each of the 3 summaries:
    lci1 <- dfCols_function(which_rows = which_rows,
                            summ = summ1,
                            colName = 'Lower 0.95')
    if(!is.null(summ2)){
       lci2 <- dfCols_function(which_rows = which_rows,
                               summ = summ2,
                               colName = 'Lower 0.95')
       names(lci2) <- rwnmes2
#       summ2Nmes <- c('t_lab_yrs',predNme,'age','year','cd4n','log10')
       cols2Choose <- grepl(paste(summ2Nmes,collapse='*|'),names(lci2))
       lci2 <- lci2[cols2Choose]
    }

    if(!is.null(summ3)){
       lci3 <- dfCols_function(which_rows = which_rows,
                               summ = summ3,
                               colName = 'Lower 0.95')
       names(lci3) <- rwnmes3
#       summ3Nmes <- c('cd4n')
       cols3Choose <- grepl(paste(summ3Nmes,collapse='*|'),names(lci3))
       lci3 <- lci3[cols3Choose]
    }

   # LCIs with spaces inserted:
   lciWspace <- vector('character',length(covariate))
   lciWspace[which(spaceHolder == 1)] <- lci1
   if(!is.null(summ2)){
      lciWspace[which(spaceHolder == 2)] <- lci2
   }

   if(!is.null(summ3)){
      lciWspace[which(spaceHolder == 3)] <- lci3
   }

   # Extracting UCIs from each of the 3 summaries:
    uci1 <- dfCols_function(which_rows = which_rows,
                            summ = summ1,
                            colName = 'Upper 0.95')
    if(!is.null(summ2)){
       uci2 <- dfCols_function(which_rows = which_rows,
                               summ = summ2,
                               colName = 'Upper 0.95')
       names(uci2) <- rwnmes2
#       summ2Nmes <- c('t_lab_yrs',predNme,'age','year','cd4n','log10')
       cols2Choose <- grepl(paste(summ2Nmes,collapse='*|'),names(uci2))
       uci2 <- uci2[cols2Choose]
    }

    if(!is.null(summ3)){
       uci3 <- dfCols_function(which_rows = which_rows,
                               summ = summ3,
                               colName = 'Upper 0.95')
       names(uci3) <- rwnmes3
#       summ3Nmes <- c('cd4n')
       cols3Choose <- grepl(paste(summ3Nmes,collapse='*|'),names(uci3))
       uci3 <- uci3[cols3Choose]
    }

   # UCIs with spaces inserted:
   uciWspace <- vector('character',length(covariate))
   uciWspace[which(spaceHolder == 1)] <- uci1
   if(!is.null(summ2)){
      uciWspace[which(spaceHolder == 2)] <- uci2
   }
   if(!is.null(summ3)){
      uciWspace[which(spaceHolder == 3)] <- uci3
   }

   # Combining LCI and UCI:
    ciWspace <- paste0('(',lciWspace, ', ',uciWspace,')')
    ciWspace[ciWspace == '(, )'] <- ''


    # P-values:
    anova_mod <- anova(mod)[,'P']
    p_rows <- NULL
    for(i in 1:length(pOrder)){
       p_rows <- c(p_rows, grep(pOrder[i],names(anova_mod),value=FALSE))
    }
    anova_p <- anova_mod[p_rows]
    anova_p <- ifelse(anova_p < 0.001,'< 0.001',
                      ifelse(anova_p >= 0.001 & anova_p <= 0.01, format(round(anova_p,3),nsmall=3),format(round(anova_p,2),nsmall=2)))
    pDF <- vector('character',length=length(covariate))
    pDF[which(pHolder==1)] <- anova_p

    df <- data.frame('Covariate' = covariate,
                     'Hazard Ratio' = hrWspace,
                     '95\\% CI' = ciWspace,
                     'P' = pDF,
                     check.names=FALSE)
    names(df)[2] <- hrNme

    return(df)

}

dfCols_function <- function(which_rows,
                            summ,
                            colName # Can be 'Effect','Lower 0.95', or 'Upper 0.95'
                            ){

                    format(round(summ[which_rows,colName],2),nsmall=2)

                   }

extractFromSummary <- function(summList, hcExp,hcNme='hc_use'){
   logOddsHC <- summList[[hcExp]][grep(hcNme,rownames(summList[[hcExp]])),'Effect']
   oddsRHC <- summList[[hcExp]][grep(hcNme,rownames(summList[[hcExp]]))+1,'Effect']
   seHC <- summList[[hcExp]][grep(hcNme,rownames(summList[[hcExp]])),'S.E.']
   lciHC <- summList[[hcExp]][grep(hcNme,rownames(summList[[hcExp]]))+1,'Lower 0.95']
   uciHC <- summList[[hcExp]][grep(hcNme,rownames(summList[[hcExp]]))+1,'Upper 0.95']

   return(list(logOddsHC = logOddsHC,
               oddsRHC = oddsRHC,
               seHC = seHC,
               lciHC = lciHC,
               uciHC = uciHC))
}
