dataPrep <- function(df,
                     eventAge,
                     maxEventAge,
                     event,
                     prevalentEvent){

   # Excluding subjects who had prevalent cases of the outcome of interest (prevalent based on first visit):
   condPrev <- df[[prevalentEvent]] == 1
   d <- df[!condPrev,]

#   # Excluding pregnancy records:
#   d <- d[d$during_pregnancy == 0,]

#   # Creating an age_at_first_record for those who were pregnant at their first visit:
#   d <- d[order(d$cfar_pid,d$age),]
#   d$age_at_first_record <- NA
#   split(d$age_at_first_record, d$cfar_pid) <- lapply(split(d$age, d$cfar_pid), FUN=function(y){
#                                                  y[1] })

   # Now that there is a new first record, must also exclude those who had an event while pregnant if pregnant at first visit:
   condNewPrev <- !is.na(d[[eventAge]]) & d[[eventAge]] <= d$age_at_first_record
   d <- d[!condNewPrev,]


   # Added 3/26/2022
   # Also need to check which, if any, cd4_or_vl_12month_gap == 1 when an event happened
   condRecordDuringLabGap <- !is.na(df$cd4_or_vl_12month_gap_start_age) &  !is.na(df$cd4_or_vl_12month_gap_stop_age) &
                             df$age >= df$cd4_or_vl_12month_gap_start_age & df$age <= df$cd4_or_vl_12month_gap_stop_age
   condEventDuringLabGap <- !is.na(df$cd4_or_vl_12month_gap_start_age) & !is.na(df$cd4_or_vl_12month_gap_stop_age) &
                             df[[eventAge]] >= df$cd4_or_vl_12month_gap_start_age &
                             df[[eventAge]] <= df$cd4_or_vl_12month_gap_stop_age
   # Some were missing so filling those in with FALSE:
   condEventDuringLabGap[which(is.na(condEventDuringLabGap))] <- FALSE
   # No CVD/thrombotic or metabolic disorder events during lab gaps so no need to adjust the eventAge to missing or anything
   # Do I just now need to remove those records where the lab gap occurred?

   #############################################################


   # Need to reorder month variable:
   d$monthOriginal <- d$month
   d$month <- NA
   split(d$month, d$cfar_pid) <- lapply(split(d$cfar_pid, d$cfar_pid), FUN=function(y){
                                    seq(1,length(y)) })

   # Subsetting records so that those having the event are subsetted at the age of the event or right after a pregnancy if the event occurred
   # during a pregnancy:
   d$maxAge <- NA
   split(d$maxAge, d$cfar_pid) <- lapply(split(d[,c(eventAge,maxEventAge)],d$cfar_pid), FUN=function(y){
                                     if(any(!is.na(y[[eventAge]]) | !is.na(y[[maxEventAge]]))){
                                       tmp <- max(c(y[[eventAge]],y[[maxEventAge]]),na.rm=TRUE)
                                     } else {
                                       tmp <- NA
                                     }
                                     return(tmp) })
   condEvent <- is.na(d[[eventAge]]) |
#                (!is.na(d[[eventAge]]) & d$age <= d[[eventAge]])
                (!is.na(d[[eventAge]]) & d$age <= d$maxAge)
   d <- d[condEvent,]


   # Also want to create overall HC use indicators (not time-varying ones) and cumYrsHCUse***Max for each dataList
   # Will use '_sub' to differentiate from those in the df data frame
   # New HC use indicators overall:
   d$hc_use_any_sub <- NA
   split(d$hc_use_any_sub, d$cfar_pid) <- lapply(split(d$hc_use_any_indicator, d$cfar_pid), FUN=function(y){
                                             max(y,na.rm=TRUE) })
   d$hc_use_any_sub.factor <- factor(d$hc_use_any_sub, levels=c(0,1), labels=c('No HC Use','HC Use'))

   d$hc_use_oral_sub <- NA
   split(d$hc_use_oral_sub, d$cfar_pid) <- lapply(split(d$hc_use_oral_indicator, d$cfar_pid), FUN=function(y){
                                             max(y,na.rm=TRUE) })
   d$hc_use_oral_sub.factor <- factor(d$hc_use_oral_sub, levels=c(0,1), labels=c('No oral HC Use','Oral HC Use'))

   d$hc_use_depo_sub <- NA
   split(d$hc_use_depo_sub, d$cfar_pid) <- lapply(split(d$hc_use_depo_indicator, d$cfar_pid), FUN=function(y){
                                             max(y,na.rm=TRUE) })
   d$hc_use_depo_sub.factor <- factor(d$hc_use_depo_sub, levels=c(0,1), labels=c('No Depo Use','Depo Use'))

   # Max cumulative Yrs on HC:
   d$cumYrsHCUseAnyMaxSub <- NA
   split(d$cumYrsHCUseAnyMaxSub, d$cfar_pid) <- lapply(split(d$cumYrsHCUseAny, d$cfar_pid), FUN=function(y){
                                                   max(y, na.rm=TRUE) })

   d$cumYrsHCUseOralMaxSub <- NA
   split(d$cumYrsHCUseOralMaxSub, d$cfar_pid) <- lapply(split(d$cumYrsHCUseOral, d$cfar_pid), FUN=function(y){
                                                   max(y, na.rm=TRUE) })

   d$cumYrsHCUseDepoMaxSub <- NA
   split(d$cumYrsHCUseDepoMaxSub, d$cfar_pid) <- lapply(split(d$cumYrsHCUseDepo, d$cfar_pid), FUN=function(y){
                                                   max(y, na.rm=TRUE) })



   return(d)
}




