### Added 4/12/2022
# Removing ART records where the regimen was started > 45 years of age
condARTGT45Start <- !is.na(art$age_at_medstart) & art$age_at_medstart > 45.0
art <- art[!condARTGT45Start,]
# Will go ahead and reset age_at_medstop to be 45 for those whose ART lasted longer than 45 years of age:
condARTGT45Stop <- !is.na(art$age_at_medstop) & art$age_at_medstop > 45.0
art$age_at_medstop[condARTGT45Stop] <- 45.0
############################
# Interested in classifying ART as PI-use, II-use, or Other
art$pi_use <- as.numeric(art$pi > 0)
art$insti_use <- as.numeric(art$ii > 0 & art$pi == 0)
art$other_use <- as.numeric(art$pi_use == 0 & art$insti_use == 0)

art$art_use <- ifelse(art$pi_use == 1, 0,
                      ifelse(art$insti_use == 1,1,
                      ifelse(art$other_use == 1,2,NA)))
art$art_use.factor <- factor(art$art_use, levels=c(0,1,2), labels=c('PI-use','INSTI-use','Other ART use'))
label(art$art_use.factor) <- 'ART Use'

# Questions:
# 1)  Review with Jamison and Jessie rules used for ART category to verify
# 2)  Is it possible to have boosted regimens that are not PI-boosted?  e.g., STRIBILLD and GENVOYA
# 3)  There are a few INSTIs that are listed as non-HAART.  Is that possible?
#     They are:  "TIVICAY", "ISENTRESS VIRAMUNE", "ISENTRESS"

# Need to compare to age at first visit:
art$age_at_first_visit <- patients[match(art$cfar_pid, patients$cfar_pid, nomatch=NA),'age_at_first_visit']
# First removing records where the ART regimen was stopped on or before the first visit:
condB4 <- art$age_at_medstop <= art$age_at_first_visit
art <- art[!condB4,]

# Indicating baseline ART use:
art$art_at_baseline <- ifelse(!is.na(art$age_at_medstart) & !is.na(art$age_at_first_visit) & art$age_at_medstart < art$age_at_first_visit &
                              !is.na(art$age_at_medstop) & !is.na(art$age_at_first_visit) & art$age_at_medstop > art$age_at_first_visit,art$art_use,NA)
art$art_at_baseline <- art$art_at_baseline + 1
art$art_at_baseline[which(is.na(art$art_at_baseline))] <- 0
art$art_at_baseline.factor <- factor(art$art_at_baseline, 
                                     levels=c(0,1,2,3),
                                     labels=c('No ART',
                                              'PI-use',
                                              'INSTI-use',
                                              'Other ART use'))

# Removing regimens that are prior to age 18:
art_original <- art
art <- art[!is.na(art$age_at_medstart) & art$age_at_medstart >= 18.0,]
art <- art[order(art$cfar_pid, art$age_at_medstart),]

art$art_number <- NA
split(art$art_number, art$cfar_pid) <- lapply(split(art$cfar_pid, art$cfar_pid), FUN=function(y){
                                          seq(1,length(y))})

art$max_art_number <- NA
split(art$max_art_number, art$cfar_pid) <- lapply(split(art$art_number, art$cfar_pid), FUN=max)

art$days_on_regimen <- 365.25*(art$age_at_medstop - art$age_at_medstart)

art$number_lt_10 <- NA
split(art$number_lt_10, art$cfar_pid) <- lapply(split(art$days_on_regimen, art$cfar_pid), FUN=function(y){
                                            length(which(y < 10)) })
tm <- unique(art$cfar_pid[which(art$number_lt_10 >= 5)])

names(art)[which(names(art) == 'age_at_medstart')] <- 'age_at_regstart'
names(art)[which(names(art) == 'age_at_medstop')] <- 'age_at_regstop'

# Updating the start and stop ages if they cross over either first visit or last censored age:

art$age_at_regstart_updated <- ifelse(art$age_at_regstart < art$age_at_first_visit &
                                      art$age_at_regstop > art$age_at_first_visit,
                                      art$age_at_first_visit,
                                      art$age_at_regstart)
art$age_at_regstart_updated2 <- ifelse(art$age_at_regstart < art$age_at_first_visit &
                                      art$age_at_regstop >= art$age_at_first_visit,
                                      art$age_at_first_visit,
                                      art$age_at_regstart)


wideVars <- Cs(cfar_pid,
               art_number,
               age_at_regstart,
               age_at_regstop,
               regimen,
               days_on_regimen,
               art_use.factor,
               haart)

art_wide <- reshape(art[,wideVars], v.names=c('age_at_regstart',
                                              'age_at_regstop',
                                              'regimen',
                                              'days_on_regimen',
                                              'art_use.factor',
                                              'haart'),
                    idvar='cfar_pid',
                    timevar = 'art_number',
                    direction = 'wide')

