cvdDx$age_at_first_visit <- patients[match(cvdDx$cfar_pid, patients$cfar_pid, nomatch=NA),'age_at_first_visit']
cvdDx$age_at_censor <- patients[match(cvdDx$cfar_pid, patients$cfar_pid, nomatch=NA),'age_at_censor']

condAfterCensor <- !is.na(cvdDx$age_at_dx_onset_2018_qtr4) & cvdDx$age_at_dx_onset_2018_qtr4 > cvdDx$age_at_censor
cvdDx <- cvdDx[!condAfterCensor,]

# ---------------------------- #
#         CVD outcomes         #
# ---------------------------- #
# Include:
# 1) Myocardial infarction
# 2) Coronary artery disease
# 3) Peripheral vascular disease
# 4) Transient ischemic events
# 5) Cerebrovascular disease -- WHAT TO USE FOR THIS??? IS THIS CVA???
# 6) Hypertension
# 7) Heart failure
# Using na_accord_diagnosis_label variable to determine
condMI <- !is.na(cvdDx$na_accord_diagnosis_label) & grepl('MI',cvdDx$na_accord_diagnosis_label) &
          !grepl('non- ',cvdDx$na_accord_diagnosis_label)
condCAD <- !is.na(cvdDx$na_accord_diagnosis_label) & grepl('CAD',cvdDx$na_accord_diagnosis_label)
condPVD <- !is.na(cvdDx$na_accord_diagnosis_label) & grepl('Peripheral',cvdDx$na_accord_diagnosis_label)
condTIA <- !is.na(cvdDx$na_accord_diagnosis_label) & grepl('Transient',cvdDx$na_accord_diagnosis_label)
condCVD <- !is.na(cvdDx$na_accord_diagnosis_label) & grepl('CVA',cvdDx$na_accord_diagnosis_label)
condHTN <- !is.na(cvdDx$na_accord_diagnosis_label) & grepl('Hypertension',cvdDx$na_accord_diagnosis_label)
condHF <- !is.na(cvdDx$diagnosis) & grepl('HEART FAILURE',cvdDx$diagnosis)

cvdDx <- diagnosis_function(d = cvdDx,
                            condNme = condMI,
                            dxNme = 'mi_dx_overall',
                            minAgeNme = 'min_age_mi_dx_overall',
                            ageVar = 'age_at_dx_onset_2018_qtr4')

cvdDx <- diagnosis_function(d = cvdDx,
                            condNme = condCAD,
                            dxNme = 'cad_dx_overall',
                            minAgeNme = 'min_age_cad_dx_overall',
                            ageVar = 'age_at_dx_onset_2018_qtr4')

cvdDx <- diagnosis_function(d = cvdDx,
                            condNme = condPVD,
                            dxNme = 'pvd_dx_overall',
                            minAgeNme = 'min_age_pvd_dx_overall',
                            ageVar = 'age_at_dx_onset_2018_qtr4')

cvdDx <- diagnosis_function(d = cvdDx,
                            condNme = condTIA,
                            dxNme = 'tia_dx_overall',
                            minAgeNme = 'min_age_tia_dx_overall',
                            ageVar = 'age_at_dx_onset_2018_qtr4')

cvdDx <- diagnosis_function(d = cvdDx,
                            condNme = condCVD,
                            dxNme = 'cvd_dx_overall',
                            minAgeNme = 'min_age_cvd_dx_overall',
                            ageVar = 'age_at_dx_onset_2018_qtr4')

cvdDx <- diagnosis_function(d = cvdDx,
                            condNme = condHTN,
                            dxNme = 'htn_dx_overall',
                            minAgeNme = 'min_age_htn_dx_overall',
                            ageVar = 'age_at_dx_onset_2018_qtr4')

cvdDx <- diagnosis_function(d = cvdDx,
                            condNme = condHF,
                            dxNme = 'hf_dx_overall',
                            minAgeNme = 'min_age_hf_dx_overall',
                            ageVar = 'age_at_dx_onset_2018_qtr4')

## Creating indicators for each of the CVD events:
#cvdInd <- function(d, cond, indNme){
#   d[[indNme]] <- 0
#   d[[indNme]][cond] <- 1
#
#   nMe <- paste0(indNme,'_overall')
#   d[[nMe]] <- NA
#   split(d[[nMe]], d$cfar_pid) <- lapply(split(d[[indNme]], d$cfar_pid), FUN=function(y){
#                                     if(any(!is.na(y) & y > 0)){
#                                        tmp <- max(y, na.rm=TRUE)
#                                     } else {
#                                        tmp <- 0
#                                     }
#                                     return(tmp)
#                                  })
#   return(d)
#}

#cvdAge <- function(d, cond, minAgeNme){
#   d$cond <- cond
#   d[[minAgeNme]] <- NA
#   split(d[[minAgeNme]], d$cfar_pid) <- lapply(split(d[,c('cond','age_at_dx_onset_2018_qtr4')], d$cfar_pid), FUN=function(y){
#                                           if(any(y[['cond']] == TRUE)){
#                                              tmp <- min(y[which(y[['cond']] == 1),'age_at_dx_onset_2018_qtr4'], na.rm=TRUE)
#                                           } else {
#                                              tmp <- NA
#                                           }
#                                           return(tmp)
#                                        })
#   return(d)
#}

## Creating indicators for each condition for CVD dx:
#cvdDx <- as.data.frame(cvdDx)
#
#cvdDx <- cvdInd(d = cvdDx,
#                cond = condMI,
#                indNme = 'mi_dx')
#
#cvdDx <- cvdInd(d = cvdDx,
#                cond = condCAD,
#                indNme = 'cad_dx')
#cvdDx <- cvdInd(d = cvdDx,
#                cond = condPVD,
#                indNme = 'pvd_dx')
#cvdDx <- cvdInd(d = cvdDx,
#                cond = condTIA,
#                indNme = 'tia_dx')
#cvdDx <- cvdInd(d = cvdDx,
#                cond = condCVD,
#                indNme = 'cvd_dx')
#cvdDx <- cvdInd(d = cvdDx,
#                cond = condHTN,
#                indNme = 'htn_dx')
#cvdDx <- cvdInd(d = cvdDx,
#                cond = condHF,
#                indNme = 'hf_dx')

## Finding minimum age at which each condition for CVD dx occurred:
#cvdDx <- cvdAge(d = cvdDx,
#                cond = condMI,
#                minAgeNme = 'min_age_mi_dx')
#cvdDx <- cvdAge(d = cvdDx,
#                cond = condCAD,
#                minAgeNme = 'min_age_cad_dx')
#cvdDx <- cvdAge(d = cvdDx,
#                cond = condPVD,
#                minAgeNme = 'min_age_pvd_dx')
#cvdDx <- cvdAge(d = cvdDx,
#                cond = condTIA,
#                minAgeNme = 'min_age_tia_dx')
#cvdDx <- cvdAge(d = cvdDx,
#                cond = condCVD,
#                minAgeNme = 'min_age_cvd_dx')
#cvdDx <- cvdAge(d = cvdDx,
#                cond = condHTN,
#                minAgeNme = 'min_age_htn_dx')
#cvdDx <- cvdAge(d = cvdDx,
#                cond = condHF,
#                minAgeNme = 'min_age_hf_dx')
#cvdDx <- as.data.table(cvdDx)
#

cvdDx$cvd_event_tmp <- as.numeric(condMI | condCAD | condPVD | condTIA | condCVD | condHTN | condHF)
cvdDx$cvd_event_age_tmp <- NA
cvdDx$cvd_event_age_tmp[condMI | condCAD | condPVD | condTIA | condCVD | condHTN | condHF] <-
    cvdDx$age_at_dx_onset_2018_qtr4[condMI | condCAD | condPVD | condTIA | condCVD | condHTN | condHF]

cvdDx$cvd_event_overall <- NA
split(cvdDx$cvd_event_overall, cvdDx$cfar_pid) <- lapply(split(cvdDx$cvd_event_tmp, cvdDx$cfar_pid), FUN=function(y){
                                                       if(any(!is.na(y))){
                                                          tmp <- as.numeric(any(y > 0))
                                                       } else {
                                                          tmp <- NA
                                                       }
                                                       return(tmp)
                                                    })

cvdDx$cvd_event_overall_age <- NA
split(cvdDx$cvd_event_overall_age, cvdDx$cfar_pid) <- lapply(split(cvdDx$cvd_event_age_tmp, cvdDx$cfar_pid), FUN=function(y){
                                                           if(any(!is.na(y))){
                                                              tmp <- min(y, na.rm=TRUE)
                                                           } else {
                                                              tmp <- NA
                                                           }
                                                           return(tmp)
                                                        })

cvdEventAges <- paste0('min_age_',Cs(mi, cad, pvd, tia, cvd, htn, hf),'_dx_overall')

cvdDx$cvd_event_overall_type <- NA
split(cvdDx$cvd_event_overall_type, cvdDx$cfar_pid) <- lapply(split(cvdDx[,c('cvd_event_overall_age',cvdEventAges)],cvdDx$cfar_pid), FUN=function(y){
                                                          if(all(!is.na(y[[1]]))){
                                                            tmp <- paste(as.numeric(which(y[1,-1] == y[1,1])), collapse=',')
                                                          } else {
                                                            tmp <- NA
                                                          }
                                                          return(tmp)
                                                       })
cvdDx$cvd_event_overall_type.factor <- factor(cvdDx$cvd_event_overall_type,
                                              levels=c(seq(1,length(cvdEventAges)),'4,5','5,6'),
                                              labels=c('MI event',
                                                       'CAD event',
                                                       'PVD event',
                                                       'TIA event',
                                                       'CVD event',
                                                       'HTN event',
                                                       'HF event',
                                                       'TIA and CVD event',
                                                       'CVD and HTN event'))

# -------------------------------------- #
#        Determining NAFLD/NASH          #
# -------------------------------------- #
cvdDx <- diagnosis_function(d = cvdDx,
                            condNme = cvdDx$na_accord_diagnosis_label == 'Non-alcoholic fatty liver disease (NAFLD / NASH)',
                            dxNme = 'nafld_dx_overall',
                            minAgeNme = 'min_age_nafld_dx_overall',
                            ageVar = 'age_at_dx_onset_2018_qtr4')
cvdDx$nafld_dx_prior_to_first_visit <- as.numeric(!is.na(cvdDx$min_age_nafld_dx_overall) &
                                                  cvdDx$min_age_nafld_dx_overall <= cvdDx$age_at_first_visit + 30/365.25)

