#labsO data management:
labsO$age_at_first_visit <- patients[match(labsO$cfar_pid, patients$cfar_pid, nomatch=NA),'age_at_first_visit']
labsO$age_at_censor <- patients[match(labsO$cfar_pid, patients$cfar_pid, nomatch=NA),'age_at_censor']
labsO$age_at_first_visit_original <- patients[match(labsO$cfar_pid, patients$cfar_pid, nomatch=NA),'age_at_first_visit_original']

condAfterCensor <- !is.na(labsO$age_at_result_date) & labsO$age_at_result_date > labsO$age_at_censor
labsO <- labsO[!condAfterCensor,]

# -------------------- #
#       Obesity        #
# -------------------- #
condBMI <- labsO$testname == 'BMI'
bmiD <- labsO[condBMI,]

bmiD <- diagnosis_function(d = bmiD,
                           condNme = !is.na(bmiD$result_numeric) & bmiD$result_numeric >= 30.0,
                           dxNme = 'obese_dx_overall',
                           minAgeNme = 'min_age_obese_dx',
                           ageVar = 'age_at_result_date')

labsO$obese_dx_overall <- bmiD[match(labsO$cfar_pid, bmiD$cfar_pid, nomatch=NA),'obese_dx_overall']
labsO$min_age_obese_dx <- bmiD[match(labsO$cfar_pid, bmiD$cfar_pid, nomatch=NA),'min_age_obese_dx']

# -------------------- #
#     Baseline BMI     #
# -------------------- #
labsO <- baseline_value_function(d = labsO,
                                condNme = labsO$testname == 'BMI',
                                minLim = bmiWindow[1],
                                maxLim = bmiWindow[2],
                                baseNme = 'bmi_at_first_visit',
                                baseAgeNme = 'age_bmi_at_first_visit')

labsO <- baseline_value_function(d = labsO,
                                condNme = labsO$testname == 'BMI',
                                minLim = bmiWindow[1],
                                maxLim = bmiWindow[2],
                                firstAge = 'age_at_first_visit_original',
                                baseNme = 'bmi_at_first_visit_original',
                                baseAgeNme = 'age_bmi_at_first_visit_original')

# -------------------- #
#       HIV-1 RNA      #
# -------------------- #
labsO$result_numeric <- ifelse(labsO$testname == 'HIV-1 RNA' & labsO$result %in% c('NOT DETECTED','Not Detected'),399,labsO$result_numeric)
labsO <- baseline_value_function(d = labsO,
                                 condNme = labsO$testname == 'HIV-1 RNA',
                                 minLim = rnaWindow[1],
                                 maxLim = rnaWindow[2],
                                 baseNme = 'vl_at_first_visit',
                                 baseAgeNme = 'age_vl_at_first_visit')

labsO <- baseline_value_function(d = labsO,
                                 condNme = labsO$testname == 'HIV-1 RNA',
                                 minLim = rnaWindow[1],
                                 maxLim = rnaWindow[2],
                                 firstAge = 'age_at_first_visit_original',
                                 baseNme = 'vl_at_first_visit_original',
                                 baseAgeNme = 'age_vl_at_first_visit_original')

# -------------------- #
#         CD4          #
# -------------------- #
labsO <- baseline_value_function(d = labsO,
                                 condNme = labsO$testname == 'CD4 COUNT',
                                 minLim = cd4Window[1],
                                 maxLim = cd4Window[2],
                                 baseNme = 'cd4_at_first_visit',
                                 baseAgeNme = 'age_cd4_at_first_visit')

labsO <- baseline_value_function(d = labsO,
                                 condNme = labsO$testname == 'CD4 COUNT',
                                 minLim = cd4Window[1],
                                 maxLim = cd4Window[2],
                                 firstAge = 'age_at_first_visit_original',
                                 baseNme = 'cd4_at_first_visit_original',
                                 baseAgeNme = 'age_cd4_at_first_visit_original')

# -------------------- #
#        Weight        #
# -------------------- #
labsO <- baseline_value_function(d = labsO,
                                 condNme = labsO$testname == 'WEIGHT',
                                 minLim = weightWindow[1],
                                 maxLim = weightWindow[2],
                                 baseNme = 'weight_at_first_visit',
                                 baseAgeNme = 'age_weight_at_first_visit')

labsO <- baseline_value_function(d = labsO,
                                 condNme = labsO$testname == 'WEIGHT',
                                 minLim = weightWindow[1],
                                 maxLim = weightWindow[2],
                                 firstAge = 'age_at_first_visit_original',
                                 baseNme = 'weight_at_first_visit_original',
                                 baseAgeNme = 'age_weight_at_first_visit_original')

# -------------------- #
#         CD8          #
# -------------------- #
condCD8 <- labsO$testname == 'CD8 COUNT'
condCD4 <- labsO$testname == 'CD4 COUNT'
cd8D <- labsO[condCD8,]
cd4D <- labsO[condCD4,]
colVars <- Cs(cfar_pid,
              age_at_result_date,
              result_numeric,
              testname,
              age_at_first_visit,
              age_at_first_visit_original)
cd4cd8D <- merge(cd4D[,colVars], cd8D[,colVars], by=c('cfar_pid','age_at_result_date','age_at_first_visit','age_at_first_visit_original'), all=TRUE)
names(cd4cd8D) <- c('cfar_pid', 'age_at_result_date','age_at_first_visit','age_at_first_visit_original','result_numeric_cd4','testname_cd4','result_numeric_cd8','testname_cd8')
cd4cd8D$result_numeric <- with(cd4cd8D, result_numeric_cd4/result_numeric_cd8) # CD4/CD8 ratio

cd4cd8D <- baseline_value_function(d = cd4cd8D,
                                   condNme = TRUE,
                                   minLim = cd4cd8Window[1],
                                   maxLim = cd4cd8Window[2],
                                   baseNme = 'cd4_cd8_ratio_at_first_visit',
                                   baseAgeNme = 'age_cd4_cd8_ratio_at_first_visit')

labsO$cd4_cd8_ratio_at_first_visit <- cd4cd8D[match(labsO$cfar_pid, cd4cd8D$cfar_pid, nomatch=NA),'cd4_cd8_ratio_at_first_visit']
labsO$age_cd4_cd8_ratio_at_first_visit <- cd4cd8D[match(labsO$cfar_pid, cd4cd8D$cfar_pid, nomatch=NA),'age_cd4_cd8_ratio_at_first_visit']


cd4cd8D <- baseline_value_function(d = cd4cd8D,
                                   condNme = TRUE,
                                   minLim = cd4cd8Window[1],
                                   maxLim = cd4cd8Window[2],
                                   firstAge = 'age_at_first_visit_original',
                                   baseNme = 'cd4_cd8_ratio_at_first_visit_original',
                                   baseAgeNme = 'age_cd4_cd8_ratio_at_first_visit_original')

labsO$cd4_cd8_ratio_at_first_visit_original <- cd4cd8D[match(labsO$cfar_pid, cd4cd8D$cfar_pid, nomatch=NA),'cd4_cd8_ratio_at_first_visit_original']
labsO$age_cd4_cd8_ratio_at_first_visit_original <- cd4cd8D[match(labsO$cfar_pid, cd4cd8D$cfar_pid, nomatch=NA),'age_cd4_cd8_ratio_at_first_visit_original']
