#patients$age_at_first_visit_original <- patients$age_at_first_visit
#patients$age_at_first_visit <- patients$new_baseline_age

# There are some women who had no records except their first pregnancy so will be excluded here:
condNoFollow <- patients$age_at_first_visit >= patients$age_at_censor
patients <- patients[!condNoFollow,]

# Generating monthly data frame (assuming a month is 28 days -- the length of time for someone to be considered exposed to a hormonal contraceptive):
data <- lapply(split(patients, patients$cfar_pid), FUN=function(y){
           tmp <- seq(y[['age_at_first_visit']],
                      y[['age_at_censor']],
                      by = 28/365.25)
           if(tmp[length(tmp)] != y[['age_at_censor']]){
#              tmp <- c(tmp, y[['age_at_lastvisit']])
               tmp <- c(tmp, y[['age_at_censor']])
           }
           tmpDf <- data.frame('cfar_pid' = rep(y[['cfar_pid']], length(tmp)),
                               'age' = tmp,
                               check.names=FALSE)
           max_age <- max(tmpDf$age)
           censor_minus_age <- y[['age_at_censor']] - max_age

           tmpDf$month <- seq(1,nrow(tmpDf))
           tmpDf <- tmpDf[,c('cfar_pid','month','age')]

           return(tmpDf)
        })

df <- do.call('rbind', data)
df <- df[order(df$cfar_pid, df$age),]
df$new_baseline_age <- patients[match(df$cfar_pid, patients$cfar_pid, nomatch=NA),'new_baseline_age']
# Added 3/26/2022:
# Censoring subjects at 45 years old:
idsB4_45 <- length(unique(df$cfar_pid))
df <- df[df$age <= 45.0,]
idsAf_45 <- length(unique(df$cfar_pid))
########################################
df$max_age_followup <- NA
split(df$max_age_followup, df$cfar_pid) <- lapply(split(df$age, df$cfar_pid), FUN=max)
df$age_at_censor <- patients[match(df$cfar_pid, patients$cfar_pid, nomatch=NA),'age_at_censor']
df$max_to_censor <- df$age_at_censor - df$max_age_followup
df$hc_use_any_max_age <- hc[match(df$cfar_pid, hc$cfar_pid, nomatch=NA),'hc_use_any_max_age']

# ----------------------------- #
#    Adding in Patients data    #
# ----------------------------- #
df$age_at_first_visit_original <- patients[match(df$cfar_pid, patients$cfar_pid, nomatch=NA),'age_at_first_visit_original']
df$age_at_first_visit <- patients[match(df$cfar_pid, patients$cfar_pid, nomatch=NA),'age_at_first_visit']
df$age_at_lastvisit <- patients[match(df$cfar_pid, patients$cfar_pid, nomatch=NA),'age_at_lastvisit']
df$year_of_firstvisit <- patients[match(df$cfar_pid, patients$cfar_pid, nomatch=NA),'year_of_firstvisit']
df$age_at_death <- patients[match(df$cfar_pid, patients$cfar_pid, nomatch=NA),'age_at_death']
df$age_at_hysterectomy <- patients[match(df$cfar_pid, patients$cfar_pid, nomatch=NA),'age_at_hysterectomy']
df$age_at_btl <- patients[match(df$cfar_pid, patients$cfar_pid, nomatch=NA),'age_at_btl']
df$btl_after_censor <- patients[match(df$cfar_pid, patients$cfar_pid, nomatch=NA),'btl_after_censor']
df$hysterectomy_after_censor <- patients[match(df$cfar_pid, patients$cfar_pid, nomatch=NA),'hysterectomy_after_censor']
df$age_at_breastca <- patients[match(df$cfar_pid, patients$cfar_pid, nomatch=NA),'age_at_breastca']
df$age_at_cervicalca <- patients[match(df$cfar_pid, patients$cfar_pid, nomatch=NA),'age_at_cervicalca']
df$age_at_ovarianca <- patients[match(df$cfar_pid, patients$cfar_pid, nomatch=NA),'age_at_ovarianca']
df$birthsex <- patients[match(df$cfar_pid, patients$cfar_pid, nomatch=NA),'birthsex']
df$race <- patients[match(df$cfar_pid, patients$cfar_pid, nomatch=NA),'race']
df$race.factor <- patients[match(df$cfar_pid, patients$cfar_pid, nomatch=NA),'race.factor']
df$hispanic <- patients[match(df$cfar_pid, patients$cfar_pid, nomatch=NA),'hispanic']
df$hispanic.factor <- patients[match(df$cfar_pid, patients$cfar_pid, nomatch=NA),'hispanic.factor']
df$race_3lvl <- patients[match(df$cfar_pid, patients$cfar_pid, nomatch=NA),'race_3lvl']
df$race_3lvl.factor <- patients[match(df$cfar_pid, patients$cfar_pid, nomatch=NA),'race_3lvl.factor']
df$race_2lvl.factor <- patients[match(df$cfar_pid, patients$cfar_pid, nomatch=NA),'race_2lvl.factor']
df$risk1 <- patients[match(df$cfar_pid, patients$cfar_pid, nomatch=NA),'risk1']
df$risk2 <- patients[match(df$cfar_pid, patients$cfar_pid, nomatch=NA),'risk2']

# ----------------------------- #
#    Adding in baseline data    #
# ----------------------------- #
df$bmi_at_first_visit <- labsO[match(df$cfar_pid, labsO$cfar_pid, nomatch=NA),'bmi_at_first_visit']
df$age_bmi_at_first_visit <- labsO[match(df$cfar_pid, labsO$cfar_pid, nomatch=NA),'age_bmi_at_first_visit']
df$bmi_at_first_visit_original <- labsO[match(df$cfar_pid, labsO$cfar_pid, nomatch=NA),'bmi_at_first_visit_original']
df$age_bmi_at_first_visit_original <- labsO[match(df$cfar_pid, labsO$cfar_pid, nomatch=NA),'age_bmi_at_first_visit_original']

df$weight_at_first_visit <- labsO[match(df$cfar_pid, labsO$cfar_pid, nomatch=NA),'weight_at_first_visit']
df$age_weight_at_first_visit <- labsO[match(df$cfar_pid, labsO$cfar_pid, nomatch=NA),'age_weight_at_first_visit']
df$weight_at_first_visit_original <- labsO[match(df$cfar_pid, labsO$cfar_pid, nomatch=NA),'weight_at_first_visit_original']
df$age_weight_at_first_visit_original <- labsO[match(df$cfar_pid, labsO$cfar_pid, nomatch=NA),'age_weight_at_first_visit_original']

df$vl_at_first_visit <- labsO[match(df$cfar_pid, labsO$cfar_pid, nomatch=NA),'vl_at_first_visit']
df$log10_vl_at_first_visit <- log10(df$vl_at_first_visit)
df$age_vl_at_first_visit <- labsO[match(df$cfar_pid, labsO$cfar_pid, nomatch=NA),'age_vl_at_first_visit']

df$vl_at_first_visit_original <- labsO[match(df$cfar_pid, labsO$cfar_pid, nomatch=NA),'vl_at_first_visit_original']
df$log10_vl_at_first_visit_original <- log10(df$vl_at_first_visit_original)
df$age_vl_at_first_visit_original <- labsO[match(df$cfar_pid, labsO$cfar_pid, nomatch=NA),'age_vl_at_first_visit_original']

df$cd4_at_first_visit <- labsO[match(df$cfar_pid, labsO$cfar_pid, nomatch=NA),'cd4_at_first_visit']
df$age_cd4_at_first_visit <- labsO[match(df$cfar_pid, labsO$cfar_pid, nomatch=NA),'age_cd4_at_first_visit']

df$cd4_at_first_visit_original <- labsO[match(df$cfar_pid, labsO$cfar_pid, nomatch=NA),'cd4_at_first_visit_original']
df$age_cd4_at_first_visit_original <- labsO[match(df$cfar_pid, labsO$cfar_pid, nomatch=NA),'age_cd4_at_first_visit_original']

### Added 3/1/2022:
dfIDsB4 <- length(unique(df$cfar_pid))
condCD4NA <- is.na(df$cd4_at_first_visit)
df <- df[!condCD4NA,]

condVLNA <- is.na(df$vl_at_first_visit)
df <- df[!condVLNA,]
dfIDsAfter <- length(unique(df$cfar_pid))
#########################
# Merged by age_at_result_date as there were no CD4/CD8 values taken not at the same age but within 30 days.
# All were measured at the same age
df$cd4_cd8_ratio_at_first_visit <- labsO[match(df$cfar_pid, labsO$cfar_pid, nomatch=NA),'cd4_cd8_ratio_at_first_visit']
df$age_cd4_cd8_ratio_at_first_visit <- labsO[match(df$cfar_pid, labsO$cfar_pid, nomatch=NA),'age_cd4_cd8_ratio_at_first_visit']

df$cd4_cd8_ratio_at_first_visit_original <- labsO[match(df$cfar_pid, labsO$cfar_pid, nomatch=NA),'cd4_cd8_ratio_at_first_visit_original']
df$age_cd4_cd8_ratio_at_first_visit_original <- labsO[match(df$cfar_pid, labsO$cfar_pid, nomatch=NA),'age_cd4_cd8_ratio_at_first_visit_original']


# ---------------------------------- #
#        Pregnancy indicator         #
# ---------------------------------- #
# Merging in data from pregnancies data:
pregnancies_wide <- as.data.frame(pregnancies_wide)
df <- merge(df, pregnancies_wide, all.x=TRUE)
df <- df[order(df$cfar_pid, df$age),]

# Creating indicator for whether time was between conception and delivery of a pregnancy:
df$during_pregnancy <- apply(df[,c('age','age_at_conception.1','age_at_delivery.1',
                                         'age_at_conception.2','age_at_delivery.2',
                                         'age_at_conception.3','age_at_delivery.3',
                                         'age_at_conception.4','age_at_delivery.4')], 1, FUN=function(y){
                                y1 <- as.numeric(!is.na(y[['age_at_conception.1']]) &
                                                 !is.na(y[['age_at_delivery.1']]) &
                                                 y[['age']] >= y[['age_at_conception.1']] &
                                                 y[['age']] <= y[['age_at_delivery.1']])
                                y2 <- as.numeric(!is.na(y[['age_at_conception.2']]) &
                                                 !is.na(y[['age_at_delivery.2']]) &
                                                 y[['age']] >= y[['age_at_conception.2']] &
                                                 y[['age']] <= y[['age_at_delivery.2']])
                                y3 <- as.numeric(!is.na(y[['age_at_conception.3']]) &
                                                 !is.na(y[['age_at_delivery.3']]) &
                                                 y[['age']] >= y[['age_at_conception.3']] &
                                                 y[['age']] <= y[['age_at_delivery.3']])
                                y4 <- as.numeric(!is.na(y[['age_at_conception.4']]) &
                                                 !is.na(y[['age_at_delivery.4']]) &
                                                 y[['age']] >= y[['age_at_conception.4']] &
                                                 y[['age']] <= y[['age_at_delivery.4']])
                                yO <- ifelse(y1==1,1,
                                             ifelse(y2 == 1,2,
                                                    ifelse(y3 ==  1,3,
                                                           ifelse(y4 == 1, 4, NA))))
                                #as.numeric(y1 | y2 | y3 | y4)
                                return(yO)
                                         })
df$during_pregnancy <- ifelse(is.na(df$during_pregnancy),0,df$during_pregnancy)
df$during_pregnancy <- ifelse(!is.na(df$new_baseline_age) & df$age == df$new_baseline_age,0,df$during_pregnancy)
df$during_pregnancy_next <- NA
split(df$during_pregnancy_next, df$cfar_pid) <- lapply(split(df$during_pregnancy, df$cfar_pid), FUN=function(y){
                                                      c(y[-1],0)
                                                })

# ---------------------------------- #
#     Adding in time-varying CD4     #
# ---------------------------------- #
df$original_data <- 1
cd4D$age_at_first_visit <- df[match(cd4D$cfar_pid, df$cfar_pid, nomatch=NA),'age_at_first_visit']
cd4D_sub <- cd4D[cd4D$age_at_result_date >= cd4D$age_at_first_visit,c('cfar_pid','age_at_result_date','result_numeric','result')]
names(cd4D_sub) <- c('cfar_pid','age_at_cd4_result','cd4_numeric','cd4')

df2 <- merge(df, cd4D_sub, by.x=c('cfar_pid','age'), by.y = c('cfar_pid','age_at_cd4_result'),all=TRUE)
condNA <- is.na(df2$original_data)
df2$age_at_cd4_result <- NA
df2$age_at_cd4_result[condNA] <- df2$age[condNA]

df2$age_at_conception.1 <- unsplit(lapply(split(df2$age_at_conception.1, df2$cfar_pid), FUN=function(y){
                             L <- !is.na(y);y[c(NA,which(L))[cumsum(L)+1]]}),
                                  df2$cfar_pid)
df2$age_at_delivery.1 <- unsplit(lapply(split(df2$age_at_delivery.1, df2$cfar_pid), FUN=function(y){
                             L <- !is.na(y);y[c(NA,which(L))[cumsum(L)+1]]}),
                                  df2$cfar_pid)

df2$age_at_conception.2 <- unsplit(lapply(split(df2$age_at_conception.2, df2$cfar_pid), FUN=function(y){
                             L <- !is.na(y);y[c(NA,which(L))[cumsum(L)+1]]}),
                                  df2$cfar_pid)
df2$age_at_delivery.2 <- unsplit(lapply(split(df2$age_at_delivery.2, df2$cfar_pid), FUN=function(y){
                             L <- !is.na(y);y[c(NA,which(L))[cumsum(L)+1]]}),
                                  df2$cfar_pid)

df2$age_at_conception.3 <- unsplit(lapply(split(df2$age_at_conception.3, df2$cfar_pid), FUN=function(y){
                             L <- !is.na(y);y[c(NA,which(L))[cumsum(L)+1]]}),
                                  df2$cfar_pid)
df2$age_at_delivery.3 <- unsplit(lapply(split(df2$age_at_delivery.3, df2$cfar_pid), FUN=function(y){
                             L <- !is.na(y);y[c(NA,which(L))[cumsum(L)+1]]}),
                                  df2$cfar_pid)

df2$age_at_conception.4 <- unsplit(lapply(split(df2$age_at_conception.4, df2$cfar_pid), FUN=function(y){
                             L <- !is.na(y);y[c(NA,which(L))[cumsum(L)+1]]}),
                                  df2$cfar_pid)
df2$age_at_delivery.4 <- unsplit(lapply(split(df2$age_at_delivery.4, df2$cfar_pid), FUN=function(y){
                             L <- !is.na(y);y[c(NA,which(L))[cumsum(L)+1]]}),
                                  df2$cfar_pid)



df2$during_pregnancy2 <- apply(df2[,c('age','age_at_conception.1','age_at_delivery.1',
                                         'age_at_conception.2','age_at_delivery.2',
                                         'age_at_conception.3','age_at_delivery.3',
                                         'age_at_conception.4','age_at_delivery.4')], 1, FUN=function(y){
                                y1 <- as.numeric(!is.na(y[['age_at_conception.1']]) &
                                                 !is.na(y[['age_at_delivery.1']]) &
                                                 y[['age']] >= y[['age_at_conception.1']] &
                                                 y[['age']] <= y[['age_at_delivery.1']])
                                y2 <- as.numeric(!is.na(y[['age_at_conception.2']]) &
                                                 !is.na(y[['age_at_delivery.2']]) &
                                                 y[['age']] >= y[['age_at_conception.2']] &
                                                 y[['age']] <= y[['age_at_delivery.2']])
                                y3 <- as.numeric(!is.na(y[['age_at_conception.3']]) &
                                                 !is.na(y[['age_at_delivery.3']]) &
                                                 y[['age']] >= y[['age_at_conception.3']] &
                                                 y[['age']] <= y[['age_at_delivery.3']])
                                y4 <- as.numeric(!is.na(y[['age_at_conception.4']]) &
                                                 !is.na(y[['age_at_delivery.4']]) &
                                                 y[['age']] >= y[['age_at_conception.4']] &
                                                 y[['age']] <= y[['age_at_delivery.4']])
                                yO <- ifelse(y1==1,1,
                                             ifelse(y2 == 1,2,
                                                    ifelse(y3 ==  1,3,
                                                           ifelse(y4 == 1, 4, NA))))
                                return(yO)
                                         })

df2$during_pregnancy2 <- ifelse(is.na(df2$during_pregnancy2),0,df2$during_pregnancy2)
df2$during_pregnancy2 <- ifelse(!is.na(df2$new_baseline_age) & df2$age == df2$new_baseline_age,0,df2$during_pregnancy2)
df2$during_pregnancy_next2 <- NA
split(df2$during_pregnancy_next2, df2$cfar_pid) <- lapply(split(df2$during_pregnancy2, df2$cfar_pid), FUN=function(y){
                                                      c(y[-1],0)
                                                })

# Changed 3/26/2022 -- Not ignoring lab data during pregnancy per Bryan in last meeting:
## Since we are censoring time during pregnancy, I am assuming that means also all lab data (we also censor any events during pregnancy
## as well).  Will change cd4, cd4_numeric, and age_at_cd4_result to missing if during_pregnancy = 1
#condDuringPregnancy <- !is.na(df2$during_pregnancy2) & df2$during_pregnancy2 == 1
#df2$cd4[condDuringPregnancy] <- NA
#df2$cd4_numeric[condDuringPregnancy] <- NA
#df2$age_at_cd4_result[condDuringPregnancy] <- NA


# 994413A has no CD4 counts recorded in CD4 data:
df2$cd4_numeric <- ifelse(is.na(df2$cd4_numeric) & df2$month == 1 & !is.na(df2$cd4_at_first_visit),
                          df2$cd4_at_first_visit,df2$cd4_numeric)
df2$cd4 <- ifelse(is.na(df2$cd4) & df2$month == 1 & !is.na(df2$cd4_at_first_visit),
                          df2$cd4_at_first_visit,df2$cd4)
df2$age_at_cd4_result <- ifelse(is.na(df2$age_at_cd4_result) & df2$month == 1 & !is.na(df2$age_cd4_at_first_visit),
                                df2$age_cd4_at_first_visit,df2$age_at_cd4_result)
df2$cd4_numeric_updated <- unsplit(lapply(split(df2$cd4_numeric, df2$cfar_pid), FUN=function(y){
                             L <- !is.na(y);y[c(NA,which(L))[cumsum(L)+1]]}),
                                  df2$cfar_pid)
df2$cd4_updated <- unsplit(lapply(split(df2$cd4, df2$cfar_pid), FUN=function(y){
                             L <- !is.na(y);y[c(NA,which(L))[cumsum(L)+1]]}),
                                  df2$cfar_pid)

df2$age_at_cd4_result_updated <- unsplit(lapply(split(df2$age_at_cd4_result, df2$cfar_pid), FUN=function(y){
                                    L <- !is.na(y)
                                    y[c(NA,which(L))[cumsum(L)+1]]}),
                                    df2$cfar_pid)

df <- df2[!is.na(df2$original_data),]
df$cd4_numeric <- df$cd4 <- df$age_at_cd4_result <- NULL
df$cd4_numeric <- df$cd4_numeric_updated
df$cd4 <- df$cd4_updated
df$age_at_cd4_result <- df$age_at_cd4_result
df$cd4_numeric_updated <- df$cd4_updated <- df$age_at_cd4_result_updated <- NULL


# Adjusting a few CD4 counts based on closest but after the age listed for the record but within 1 month of it:
df$cd4_age_diff <- 365.25*(df$age - df$age_at_cd4_result)

df$age_at_cd4_result_next <- NA
split(df$age_at_cd4_result_next, df$cfar_pid) <- lapply(split(df$age_at_cd4_result, df$cfar_pid), FUN=function(y){
                                                   if(length(y) > 1){
                                                      tmp <- c(y[2:length(y)], NA)
                                                   } else {
                                                      tmp <- NA
                                                   }
                                                   return(tmp) })
df$cd4_age_diff_next <- 365.5*(df$age_at_cd4_result_next - df$age)

condInd <- which(df$cd4_age_diff > df$cd4_age_diff_next & df$cd4_age_diff_next > 0 & df$cd4_age_diff_next <= 28.0)
df$cd4_numeric[condInd] <- df$cd4_numeric[(condInd+1)]
df$cd4[condInd] <- df$cd4[(condInd+1)]
df$age_at_cd4_result[condInd] <- df$age_at_cd4_result[(condInd+1)]
df$age_at_cd4_result_next[condInd] <- df$age_at_cd4_result_next[(condInd+1)]
df$cd4_age_diff_original <- df$cd4_age_diff
df$cd4_age_diff <- 365.25*(df$age - df$age_at_cd4_result)

# ---------------------------------- #
#  Adding in time-varying HIV-1 RNA  #
# ---------------------------------- #
condVL <- labsO$testname == 'HIV-1 RNA'
rnaD <- labsO[condVL,]

rnaD$age_at_first_visit <- df[match(rnaD$cfar_pid, df$cfar_pid, nomatch=NA),'age_at_first_visit']
rnaD_sub <- rnaD[rnaD$age_at_result_date >= rnaD$age_at_first_visit,c('cfar_pid','age_at_result_date','result_numeric','result')]
names(rnaD_sub) <- c('cfar_pid','age_at_vl_result','vl_numeric','vl')

df2 <- merge(df, rnaD_sub, by.x=c('cfar_pid','age'), by.y = c('cfar_pid','age_at_vl_result'),all=TRUE)
condNA <- is.na(df2$original_data)
df2$age_at_vl_result <- NA
df2$age_at_vl_result[condNA] <- df2$age[condNA]


df2$age_at_conception.1 <- unsplit(lapply(split(df2$age_at_conception.1, df2$cfar_pid), FUN=function(y){
                             L <- !is.na(y);y[c(NA,which(L))[cumsum(L)+1]]}),
                                  df2$cfar_pid)
df2$age_at_delivery.1 <- unsplit(lapply(split(df2$age_at_delivery.1, df2$cfar_pid), FUN=function(y){
                             L <- !is.na(y);y[c(NA,which(L))[cumsum(L)+1]]}),
                                  df2$cfar_pid)

df2$age_at_conception.2 <- unsplit(lapply(split(df2$age_at_conception.2, df2$cfar_pid), FUN=function(y){
                             L <- !is.na(y);y[c(NA,which(L))[cumsum(L)+1]]}),
                                  df2$cfar_pid)
df2$age_at_delivery.2 <- unsplit(lapply(split(df2$age_at_delivery.2, df2$cfar_pid), FUN=function(y){
                             L <- !is.na(y);y[c(NA,which(L))[cumsum(L)+1]]}),
                                  df2$cfar_pid)

df2$age_at_conception.3 <- unsplit(lapply(split(df2$age_at_conception.3, df2$cfar_pid), FUN=function(y){
                             L <- !is.na(y);y[c(NA,which(L))[cumsum(L)+1]]}),
                                  df2$cfar_pid)
df2$age_at_delivery.3 <- unsplit(lapply(split(df2$age_at_delivery.3, df2$cfar_pid), FUN=function(y){
                             L <- !is.na(y);y[c(NA,which(L))[cumsum(L)+1]]}),
                                  df2$cfar_pid)

df2$age_at_conception.4 <- unsplit(lapply(split(df2$age_at_conception.4, df2$cfar_pid), FUN=function(y){
                             L <- !is.na(y);y[c(NA,which(L))[cumsum(L)+1]]}),
                                  df2$cfar_pid)
df2$age_at_delivery.4 <- unsplit(lapply(split(df2$age_at_delivery.4, df2$cfar_pid), FUN=function(y){
                             L <- !is.na(y);y[c(NA,which(L))[cumsum(L)+1]]}),
                                  df2$cfar_pid)



df2$during_pregnancy2 <- apply(df2[,c('age','age_at_conception.1','age_at_delivery.1',
                                         'age_at_conception.2','age_at_delivery.2',
                                         'age_at_conception.3','age_at_delivery.3',
                                         'age_at_conception.4','age_at_delivery.4')], 1, FUN=function(y){
                                y1 <- as.numeric(!is.na(y[['age_at_conception.1']]) &
                                                 !is.na(y[['age_at_delivery.1']]) &
                                                 y[['age']] >= y[['age_at_conception.1']] &
                                                 y[['age']] <= y[['age_at_delivery.1']])
                                y2 <- as.numeric(!is.na(y[['age_at_conception.2']]) &
                                                 !is.na(y[['age_at_delivery.2']]) &
                                                 y[['age']] >= y[['age_at_conception.2']] &
                                                 y[['age']] <= y[['age_at_delivery.2']])
                                y3 <- as.numeric(!is.na(y[['age_at_conception.3']]) &
                                                 !is.na(y[['age_at_delivery.3']]) &
                                                 y[['age']] >= y[['age_at_conception.3']] &
                                                 y[['age']] <= y[['age_at_delivery.3']])
                                y4 <- as.numeric(!is.na(y[['age_at_conception.4']]) &
                                                 !is.na(y[['age_at_delivery.4']]) &
                                                 y[['age']] >= y[['age_at_conception.4']] &
                                                 y[['age']] <= y[['age_at_delivery.4']])
                                yO <- ifelse(y1==1,1,
                                             ifelse(y2 == 1,2,
                                                    ifelse(y3 ==  1,3,
                                                           ifelse(y4 == 1, 4, NA))))
                                return(yO)
                                         })

df2$during_pregnancy2 <- ifelse(is.na(df2$during_pregnancy2),0,df2$during_pregnancy2)
df2$during_pregnancy2 <- ifelse(!is.na(df2$new_baseline_age) & df2$age == df2$new_baseline_age,0,df2$during_pregnancy2)
df2$during_pregnancy_next2 <- NA
split(df2$during_pregnancy_next2, df2$cfar_pid) <- lapply(split(df2$during_pregnancy2, df2$cfar_pid), FUN=function(y){
                                                      c(y[-1],0)
                                                })

# Changed 3/26/2022:  Not to ignore lab data during pregnancy per Bryan's request at last meeting:
## Since we are censoring time during pregnancy, I am assuming that means also all lab data (we also censor any events during pregnancy
## as well).  Will change cd4, cd4_numeric, and age_at_cd4_result to missing if during_pregnancy = 1
#condDuringPregnancy <- !is.na(df2$during_pregnancy2) & df2$during_pregnancy2 == 1
#df2$vl[condDuringPregnancy] <- NA
#df2$vl_numeric[condDuringPregnancy] <- NA
#df2$age_at_vl_result[condDuringPregnancy] <- NA


df2$vl_numeric <- ifelse(is.na(df2$vl_numeric) & df2$month == 1 & !is.na(df2$vl_at_first_visit),
                          df2$vl_at_first_visit,df2$vl_numeric)
df2$vl <- ifelse(is.na(df2$vl) & df2$month == 1 & !is.na(df2$vl_at_first_visit),
                          df2$vl_at_first_visit,df2$vl)
df2$age_at_vl_result <- ifelse(is.na(df2$age_at_vl_result) & df2$month == 1 & !is.na(df2$age_vl_at_first_visit),
                                df2$age_vl_at_first_visit,df2$age_at_vl_result)
df2$vl_numeric_updated <- unsplit(lapply(split(df2$vl_numeric, df2$cfar_pid), FUN=function(y){
                             L <- !is.na(y);y[c(NA,which(L))[cumsum(L)+1]]}),
                                  df2$cfar_pid)
df2$vl_updated <- unsplit(lapply(split(df2$vl, df2$cfar_pid), FUN=function(y){
                             L <- !is.na(y);y[c(NA,which(L))[cumsum(L)+1]]}),
                                  df2$cfar_pid)

df2$age_at_vl_result_updated <- unsplit(lapply(split(df2$age_at_vl_result, df2$cfar_pid), FUN=function(y){
                                    L <- !is.na(y)
                                    y[c(NA,which(L))[cumsum(L)+1]]}),
                                    df2$cfar_pid)

df <- df2[!is.na(df2$original_data),]
df$vl_numeric <- df$vl <- df$age_at_vl_result <- NULL
df$vl_numeric <- df$vl_numeric_updated
df$vl <- df$vl_updated
df$age_at_vl_result <- df$age_at_vl_result
df$vl_numeric_updated <- df$vl_updated <- df$age_at_vl_result_updated <- NULL

# Adjusting a few viral loads based on closest but after the age listed for the record but within 1 month of it:
df$vl_age_diff <- 365.25*(df$age - df$age_at_vl_result)

df$age_at_vl_result_next <- NA
split(df$age_at_vl_result_next, df$cfar_pid) <- lapply(split(df$age_at_vl_result, df$cfar_pid), FUN=function(y){
                                                   if(length(y) > 1){
                                                      tmp <- c(y[2:length(y)], NA)
                                                   } else {
                                                      tmp <- NA
                                                   }
                                                   return(tmp) })
df$vl_age_diff_next <- 365.5*(df$age_at_vl_result_next - df$age)

condInd <- which(df$vl_age_diff > df$vl_age_diff_next & df$vl_age_diff_next > 0 & df$vl_age_diff_next <= 28.0)
df$vl_numeric[condInd] <- df$vl_numeric[(condInd+1)]
df$log10_vl_numeric <- log10(df$vl_numeric)
df$vl[condInd] <- df$vl[(condInd+1)]
df$age_at_vl_result[condInd] <- df$age_at_vl_result[(condInd+1)]
df$age_at_vl_result_next[condInd] <- df$age_at_vl_result_next[(condInd+1)]

df$vl_age_diff_original <- df$vl_age_diff
df$vl_age_diff <- 365.25*(df$age - df$age_at_vl_result)

# Added 3/26/2022:
# Creating indicators for whether cd4_age_diff > 365.25 or vl_age_diff > 365.25
df$cd4_or_vl_12month_gap <- as.numeric((!is.na(df$cd4_age_diff) & df$cd4_age_diff > 365.25) |
                                       (!is.na(df$vl_age_diff) & df$vl_age_diff > 365.25))
df$cd4_or_vl_prev_gap_ind <- NA
split(df$cd4_or_vl_prev_gap_ind, df$cfar_pid) <- lapply(split(df$cd4_or_vl_12month_gap, df$cfar_pid), FUN=function(y){
                                                    tmp <- c(NA,y[-length(y)])
                                                    return(tmp) })
df$cd4_or_vl_next_gap_ind <- NA
split(df$cd4_or_vl_next_gap_ind, df$cfar_pid) <- lapply(split(df$cd4_or_vl_12month_gap, df$cfar_pid), FUN=function(y){
                                                    if(length(y) > 1){
                                                       tmp <- c(y[-1],NA)
                                                    } else {
                                                       tmp <- NA
                                                    }
                                                    return(tmp) })

#df$cd4_or_vl_12month_gap_start_age <- ifelse(df$cd4_or_vl_12month_gap == 1 &
#                                             df$cd4_or_vl_prev_gap_ind == 0,df$age,NA)
df$cd4_or_vl_12month_gap_lag_diff <- NA
split(df$cd4_or_vl_12month_gap_lag_diff, df$cfar_pid) <- lapply(split(df$cd4_or_vl_12month_gap, df$cfar_pid), FUN=function(y){
                                                            if(length(y) > 1){
                                                               tmp <- c(diff(y),NA)
                                                            } else {
                                                               tmp <- 0
                                                            } })
df$cd4_or_vl_12month_gap_start_age <- ifelse(df$cd4_or_vl_12month_gap == 0 &
                                             df$cd4_or_vl_12month_gap_lag_diff == 1,df$age,NA)

# Using LOCF to fill in start age of lab gaps:
df$cd4_or_vl_12month_gap_start_age <- unsplit(lapply(split(df$cd4_or_vl_12month_gap_start_age,
                                                           df$cfar_pid), FUN=function(y){
                                                        L <- !is.na(y);y[c(NA,which(L))[cumsum(L)+1]]}),
                                              df$cfar_pid)

df$cd4_or_vl_12month_gap_stop_age <- ifelse(df$cd4_or_vl_12month_gap == 1 &
                                             df$cd4_or_vl_next_gap_ind == 0,df$age,NA)

df <- df[order(df$cfar_pid, df$age, decreasing=TRUE),]
df$cd4_or_vl_12month_gap_stop_age <- unsplit(lapply(split(df$cd4_or_vl_12month_gap_stop_age,
                                                           df$cfar_pid), FUN=function(y){
                                                        L <- !is.na(y);y[c(NA,which(L))[cumsum(L)+1]]}),
                                              df$cfar_pid)
df <- df[order(df$cfar_pid, df$age, decreasing=TRUE),]
df$cd4_or_vl_12month_gap_stop_age <- ifelse(is.na(df$cd4_or_vl_12month_gap_start_age),NA,
                                            ifelse(!is.na(df$cd4_or_vl_12month_gap_start_age) &
                                                   df$cd4_or_vl_12month_gap == 0,NA,
                                                   df$cd4_or_vl_12month_gap_stop_age))
# Because we are starting censoring at the record before the first one with a difference of > 365.25, need to manually
# set that indicator to 1 as it is 0 right now:
df$cd4_or_vl_12month_gap[which(df$age == df$cd4_or_vl_12month_gap_start_age)] <- 1

# Will then evaluate whether:
# a) gap was during pregnancy -- already censored
# b) an event occurred during gap if gap was not during pregnancy
#############################################################



# Changed 3/26/2022:  No longer removing records at the start of 12 month gaps in CD4 or viral load data
# Instead will censor like pregnancy data and allow someone to come back into the dataset
## Added 3/3/2022
## Censoring when there is > 365.25 gap in either CD4 or HIV-1 RNA measurements:
#dfIDsB4Gap <- length(unique(df$cfar_pid))
#condCD4Gap <- !is.na(df$cd4_age_diff) & df$cd4_age_diff > 365.25
#condVLGap <- !is.na(df$vl_age_diff) & df$vl_age_diff > 365.25
#
#df$censor12MonthGap <- NA
#df$censor12MonthGap[condCD4Gap | condVLGap] <- 1
#
#
#df$censor12MonthGap_updated <- unsplit(lapply(split(df$censor12MonthGap, df$cfar_pid), FUN=function(y){
#                                    L <- !is.na(y)
#                                    y[c(NA,which(L))[cumsum(L)+1]]}),
#                                    df$cfar_pid)
#df <- df[-which(!is.na(df$censor12MonthGap_updated) & df$censor12MonthGap_updated == 1),]
##df <- df[!condCD4Gap & !condVLGap,]
#dfIDsAfterGap <- length(unique(df$cfar_pid))
#df$censor12MonthGap <- df$censor12MonthGap_updated <- NULL
################################

# ----------------------------- #
#      Adding in DM data        #
# ----------------------------- #
# Based on diagnosis data (addDx):
df$min_age_dm_dx <- addDx[match(df$cfar_pid, addDx$cfar_pid, nomatch=NA),'min_age_dm_dx']
df$dm_dx_overall <- addDx[match(df$cfar_pid, addDx$cfar_pid, nomatch=NA),'dm_dx_overall']
df$dm_dx_prior_to_first_visit <- addDx[match(df$cfar_pid, addDx$cfar_pid, nomatch=NA),'dm_dx_prior_to_first_visit']
#df$dm_dx_prior_to_first_visit[which(is.na(df$dm_dx_prior_to_first_visit))] <- 0
#condDMDXPrior <- !is.na(df$dm_dx_prior_to_first_visit) & df$dm_dx_prior_to_first_visit == 1
#df <- df[!condDMDXPrior,]

# Based on labsD data:
df$hba1cGT_overall <- dmD[match(df$cfar_pid, dmD$cfar_pid, nomatch=NA),'hba1cGT_overall']
#df$hba1cGT[which(is.na(df$hba1cGT))] <- 0
df$min_age_hba1c_dm <- dmD[match(df$cfar_pid, dmD$cfar_pid, nomatch=NA),'min_age_hba1c_dm']
df$dm_hba1c_prior_to_first_visit <- dmD[match(df$cfar_pid, dmD$cfar_pid, nomatch=NA),'dm_hba1c_prior_to_first_visit']
#df$dm_hba1c_prior_to_first_visit[which(is.na(df$dm_hba1c_prior_to_first_visit))] <- 0
#condDMPrior <- !is.na(df$dm_hba1c_prior_to_first_visit) & df$dm_hba1c_prior_to_first_visit == 1
#df <- df[!condDMPrior,]


df$diabetes_event_overall <- NA
split(df$diabetes_event_overall, df$cfar_pid) <- lapply(split(df[,c('dm_dx_overall','hba1cGT_overall')], df$cfar_pid),
                                                    FUN=function(y){
                                                       as.numeric(any(y > 0))
                                                    })

df$diabetes_event_overall_age <- NA
split(df$diabetes_event_overall_age, df$cfar_pid) <- lapply(split(df[,c('min_age_dm_dx','min_age_hba1c_dm')], df$cfar_pid), FUN=function(y){
                                                        if(!all(is.na(y))){
                                                           tmp <- min(y, na.rm=TRUE)
                                                        } else {
                                                           tmp <- NA
                                                        }
                                                        return(tmp)
                                                     })

# ----------------------------- #
#   Adding in smoking status    #
# ----------------------------- #
df$smoking_ever <- substanceUse[match(df$cfar_pid, substanceUse$cfar_pid, nomatch=NA),'smoking_ever']

# ----------------------------- #
#  Adding in dyslipidemia data  #
# ----------------------------- #
# From diagnosis data (addDx):
df$min_age_dyslipidemia_dx <- addDx[match(df$cfar_pid, addDx$cfar_pid, nomatch=NA),'min_age_dyslipidemia_dx']
df$dyslipidemia_dx_overall <- addDx[match(df$cfar_pid, addDx$cfar_pid, nomatch=NA),'dyslipidemia_dx_overall']
df$dyslipidemia_dx_prior_to_first_visit <- addDx[match(df$cfar_pid, addDx$cfar_pid, nomatch=NA),'dyslipidemia_dx_prior_to_first_visit']
#df$dyslipidemia_dx_prior_to_first_visit[which(is.na(df$dyslipidemia_dx_prior_to_first_visit))] <- 0

# From LDL:
df$ldlGT_overall <- ldlD[match(df$cfar_pid, ldlD$cfar_pid, nomatch=NA),'ldlGT_overall']
df$min_age_ldl_gt <- ldlD[match(df$cfar_pid, ldlD$cfar_pid, nomatch=NA),'min_age_ldl_gt']
df$ldlGT_prior_to_first_visit <- ldlD[match(df$cfar_pid, ldlD$cfar_pid, nomatch=NA),'ldlGT_prior_to_first_visit']

# From total cholesterol:
df$tcGT_overall <- cholD[match(df$cfar_pid, cholD$cfar_pid, nomatch=NA),'tcGT_overall']
df$min_age_tc_gt <- cholD[match(df$cfar_pid, cholD$cfar_pid, nomatch=NA),'min_age_tc_gt']
df$tcGT_prior_to_first_visit <- cholD[match(df$cfar_pid, cholD$cfar_pid, nomatch=NA),'tcGT_prior_to_first_visit']

# From labs data:
df$dyslipidemia_labs <- labsD[match(df$cfar_pid, labsD$cfar_pid, nomatch=NA),'dyslipidemia_labs']


df$dyslipidemia_event_overall <- NA
split(df$dyslipidemia_event_overall, df$cfar_pid) <- lapply(split(df[,c('dyslipidemia_dx_overall','dyslipidemia_labs')], df$cfar_pid),
                                                    FUN=function(y){
                                                       as.numeric(any(y > 0))
                                                    })

df$dyslipidemia_event_overall_age <- NA
split(df$dyslipidemia_event_overall_age, df$cfar_pid) <- lapply(split(df[,c('min_age_dyslipidemia_dx','min_age_ldl_gt','min_age_tc_gt')], df$cfar_pid), FUN=function(y){
                                                        if(!all(is.na(y))){
                                                           tmp <- min(y, na.rm=TRUE)
                                                        } else {
                                                           tmp <- NA
                                                        }
                                                        return(tmp)
                                                     })


# ----------------------------- #
#   Adding in NAFLD/NASH data   #
# ----------------------------- #
df$nafld_dx_overall <- cvdDx[match(df$cfar_pid, cvdDx$cfar_pid, nomatch=NA),'nafld_dx_overall']
df$min_age_nafld_dx_overall <- cvdDx[match(df$cfar_pid, cvdDx$cfar_pid, nomatch=NA),'min_age_nafld_dx_overall']
df$nafld_dx_prior_to_first_visit <- cvdDx[match(df$cfar_pid, cvdDx$cfar_pid, nomatch=NA),'nafld_dx_prior_to_first_visit']

df$nafld_event_overall <- df$nafld_dx_overall
df$nafld_event_overall_age <- df$min_age_nafld_dx_overall

# ----------------------------- #
#       Obesity information     #
# ----------------------------- #
df$obese_dx_overall <- labsO[match(df$cfar_pid, labsO$cfar_pid, nomatch=NA),'obese_dx_overall']
df$min_age_obese_dx <- labsO[match(df$cfar_pid, labsO$cfar_pid, nomatch=NA),'min_age_obese_dx']
df$obesity_event_overall <- df$obese_dx_overall
df$obesity_event_overall_age <- df$min_age_obese_dx

# ----------------------------- #
#      Metabolic disorders      #
# ----------------------------- #
df$metabolic_disorder_event_overall <- apply(df[,c('diabetes_event_overall','dyslipidemia_event_overall','nafld_event_overall','obesity_event_overall')],
                                             1,
                                             FUN=function(y){
                                                as.numeric(any(!is.na(y) & y > 0))
                                             })

df$metabolic_disorder_event_overall_age <- NA
split(df$metabolic_disorder_event_overall_age, df$cfar_pid) <- lapply(split(df[,c('diabetes_event_overall_age','dyslipidemia_event_overall_age',
                                                                                  'nafld_event_overall_age','obesity_event_overall_age')],
                                                                            df$cfar_pid),
                                                                      FUN=function(y){
                                                                         if(any(!is.na(y))){
                                                                            tmp <- min(y, na.rm=TRUE)
                                                                         } else {
                                                                            tmp <- NA
                                                                         }
                                                                         return(tmp)
                                                               })


mdEventAges <- paste0(c('diabetes','dyslipidemia','nafld','obesity'),
                        '_event_overall_age')

df$metabolic_disorder_event_overall_type <- NA
split(df$metabolic_disorder_event_overall_type, df$cfar_pid) <- lapply(split(df[,c('metabolic_disorder_event_overall_age',mdEventAges)],
                                                                             df$cfar_pid), FUN=function(y){
                                                                   if(any(!is.na(y[[1]]))){
                                                                      tmp <- paste(which(y[1,-1] == y[[1]]),collapse=',')
                                                                   } else {
                                                                      tmp <- NA
                                                                   }
                                                                   return(tmp)
                                                                })
df$metabolic_disorder_event_overall_type.factor <- factor(df$metabolic_disorder_event_overall_type,
                                                          levels=c(seq(1,4),'1,2,4','1,4','2,3','2,4'),
                                                          labels=c('Diabetes event',
                                                                   'Dyslipidemia event',
                                                                   'NAFLD event',
                                                                   'Obesity event',
                                                                   'Diabetes, dyslipidemia, and obesity event',
                                                                   'Diabetes and obesity event',
                                                                   'Dyslipidemia and NAFLD event',
                                                                   'Dyslipidemia and obesity event'))


# ----------------------------- #
#      Adding in CVD data       #
# ----------------------------- #
df <- as.data.frame(df)
cvdDx <- as.data.frame(cvdDx)
df$mi_dx_overall <- cvdDx[match(df$cfar_pid, cvdDx$cfar_pid, nomatch=NA),'mi_dx_overall']
df$cad_dx_overall <- cvdDx[match(df$cfar_pid, cvdDx$cfar_pid, nomatch=NA),'cad_dx_overall']
df$pvd_dx_overall <- cvdDx[match(df$cfar_pid, cvdDx$cfar_pid, nomatch=NA),'pvd_dx_overall']
df$tia_dx_overall <- cvdDx[match(df$cfar_pid, cvdDx$cfar_pid, nomatch=NA),'tia_dx_overall']
df$cvd_dx_overall <- cvdDx[match(df$cfar_pid, cvdDx$cfar_pid, nomatch=NA),'cvd_dx_overall']
df$htn_dx_overall <- cvdDx[match(df$cfar_pid, cvdDx$cfar_pid, nomatch=NA),'htn_dx_overall']
df$hf_dx_overall <- cvdDx[match(df$cfar_pid, cvdDx$cfar_pid, nomatch=NA),'hf_dx_overall']

df$min_age_mi_dx_overall <- cvdDx[match(df$cfar_pid, cvdDx$cfar_pid, nomatch=NA),'min_age_mi_dx_overall']
df$min_age_cad_dx_overall <- cvdDx[match(df$cfar_pid, cvdDx$cfar_pid, nomatch=NA),'min_age_cad_dx_overall']
df$min_age_pvd_dx_overall <- cvdDx[match(df$cfar_pid, cvdDx$cfar_pid, nomatch=NA),'min_age_pvd_dx_overall']
df$min_age_tia_dx_overall <- cvdDx[match(df$cfar_pid, cvdDx$cfar_pid, nomatch=NA),'min_age_tia_dx_overall']
df$min_age_cvd_dx_overall <- cvdDx[match(df$cfar_pid, cvdDx$cfar_pid, nomatch=NA),'min_age_cvd_dx_overall']
df$min_age_htn_dx_overall <- cvdDx[match(df$cfar_pid, cvdDx$cfar_pid, nomatch=NA),'min_age_htn_dx_overall']
df$min_age_hf_dx_overall <- cvdDx[match(df$cfar_pid, cvdDx$cfar_pid, nomatch=NA),'min_age_hf_dx_overall']


df$cvd_event_overall <- cvdDx[match(df$cfar_pid, cvdDx$cfar_pid, nomatch=NA),'cvd_event_overall']
df$cvd_event_overall_age <- cvdDx[match(df$cfar_pid, cvdDx$cfar_pid, nomatch=NA),'cvd_event_overall_age']
df$cvd_event_overall_type.factor <- cvdDx[match(df$cfar_pid, cvdDx$cfar_pid, nomatch=NA),'cvd_event_overall_type.factor']

# ------------------------------ #
#  Adding in thromobolysis data  #
# ------------------------------ #
df$thrombosis_event_overall <- rawDx[match(df$cfar_pid, rawDx$cfar_pid, nomatch=NA),'thrombosis_event_overall']
df$thrombosis_event_overall_age <- rawDx[match(df$cfar_pid, rawDx$cfar_pid, nomatch=NA),'thrombosis_event_overall_age']

# ---------------------------------- #
#  Creating CVD and thrombosis event #
# ---------------------------------- #
condCVDorT <- df$cvd_event_overall == 1 | df$thrombosis_event_overall == 1
df$cvd_or_thrombosis_event_overall <- as.numeric(condCVDorT)
df$cvd_or_thrombosis_event_overall_age <- apply(df[,c('cvd_event_overall','cvd_event_overall_age',
                                                      'thrombosis_event_overall','thrombosis_event_overall_age')], 1,
                                                FUN=function(y){
                                                   if((!is.na(y[['cvd_event_overall']]) & y[['cvd_event_overall']] == 1) |
                                                      (!is.na(y[['thrombosis_event_overall']]) & y[['thrombosis_event_overall']] == 1)){
                                                      tmp <- min(c(y[['cvd_event_overall_age']],y[['thrombosis_event_overall_age']]),na.rm=TRUE)
                                                   } else {
                                                      tmp <- NA
                                                   }
                                                   return(tmp)
                                          })




cvdThrombosisEventAges <- c('thrombosis_event_overall_age',paste0('min_age_',Cs(mi, cad, pvd, tia, cvd, htn, hf),'_dx_overall'))
thrombosisEventAges <- 'thrombosis_event_overall_age'

df$cvd_or_thrombosis_event_overall_type <- NA
split(df$cvd_or_thrombosis_event_overall_type, df$cfar_pid) <- lapply(split(df[,c('cfar_pid','cvd_or_thrombosis_event_overall_age',cvdThrombosisEventAges)],
                                                                            df$cfar_pid), FUN=function(y){
                                                                        if(any(!is.na(y[[2]]))){
                                                                           tmp <- paste(as.numeric(which(y[1,-c(1:2)] == y[1,2])), collapse=',')
                                                                        } else {
                                                                           tmp <- NA
                                                                        }
                                                                        return(tmp) })

df$cvd_or_thrombosis_event_overall_type.factor <- factor(df$cvd_or_thrombosis_event_overall_type,
                                                         levels=c(seq(1,length(cvdThrombosisEventAges)),
                                                                  '1,7',
                                                                  '5,6',
                                                                  '6,7'),
                                                         labels=c('Thrombosis event',
                                                                  'MI event',
                                                                  'CAD event',
                                                                  'PVD event',
                                                                  'TIA event',
                                                                  'CVD event',
                                                                  'HTN event',
                                                                  'HF event',
                                                                  'Thrombosis and HTN event',
                                                                  'TIA and CVD event',
                                                                  'CVD and HTN event'))


# --------------------------------------- #
#  Outcome events relative to first visit #
# --------------------------------------- #
outcome_age_chk <- function(d = df,
                            ageFirst = 'age_at_first_visit',
                            ageVar){
#                      if(ageFirst == 'age_at_first_visit'){
                         return(as.numeric(!is.na(df[[ageVar]]) &
                                           df[[ageVar]] < df[[ageFirst]] + 28/365.25))
#                      } else {
#                         return(as.numeric(!is.na(df[[ageVar]]) &
#                                           df[[ageVar]] < df[[ageFirst]]))
#                      }
}
df$diabetes_event_overall_prior_to_first_visit <- outcome_age_chk(ageVar='diabetes_event_overall_age')
df$cvd_event_overall_prior_to_first_visit <- outcome_age_chk(ageVar='cvd_event_overall_age')
df$metabolic_disorder_event_overall_prior_to_first_visit <- outcome_age_chk(ageVar='metabolic_disorder_event_overall_age')
df$thrombosis_event_overall_prior_to_first_visit <- outcome_age_chk(ageVar='thrombosis_event_overall_age')
df$cvd_or_thrombosis_event_overall_prior_to_first_visit <- outcome_age_chk(ageVar='cvd_or_thrombosis_event_overall_age')

# -------------------------------- #
#  Adding in prior HCV status data #
# -------------------------------- #
df$hcv_status_ever <- hcv[match(df$cfar_pid, hcv$cfar_pid, nomatch=NA),'hcv_ever']



## ----------------------------- #
##    Adding in baseline data    #
## ----------------------------- #
#df$bmi_at_first_visit <- labsO[match(df$cfar_pid, labsO$cfar_pid, nomatch=NA),'bmi_at_first_visit']
#df$age_bmi_at_first_visit <- labsO[match(df$cfar_pid, labsO$cfar_pid, nomatch=NA),'age_bmi_at_first_visit']

#df$weight_at_first_visit <- labsO[match(df$cfar_pid, labsO$cfar_pid, nomatch=NA),'weight_at_first_visit']
#df$age_weight_at_first_visit <- labsO[match(df$cfar_pid, labsO$cfar_pid, nomatch=NA),'age_weight_at_first_visit']

#df$vl_at_first_visit <- labsO[match(df$cfar_pid, labsO$cfar_pid, nomatch=NA),'vl_at_first_visit']
#df$log10_vl_at_first_visit <- log10(df$vl_at_first_visit)
#df$age_vl_at_first_visit <- labsO[match(df$cfar_pid, labsO$cfar_pid, nomatch=NA),'age_vl_at_first_visit']

#df$cd4_at_first_visit <- labsO[match(df$cfar_pid, labsO$cfar_pid, nomatch=NA),'cd4_at_first_visit']
#df$age_cd4_at_first_visit <- labsO[match(df$cfar_pid, labsO$cfar_pid, nomatch=NA),'age_cd4_at_first_visit']

#### Added 3/1/2022:
#dfIDsB4 <- length(unique(df$cfar_pid))
#condCD4NA <- is.na(df$cd4_at_first_visit)
#df <- df[!condCD4NA,]

#condVLNA <- is.na(df$vl_at_first_visit)
#df <- df[!condVLNA,]
#dfIDsAfter <- length(unique(df$cfar_pid))
##########################
## Merged by age_at_result_date as there were no CD4/CD8 values taken not at the same age but within 30 days.
## All were measured at the same age
#df$cd4_cd8_ratio_at_first_visit <- labsO[match(df$cfar_pid, labsO$cfar_pid, nomatch=NA),'cd4_cd8_ratio_at_first_visit']
#df$age_cd4_cd8_ratio_at_first_visit <- labsO[match(df$cfar_pid, labsO$cfar_pid, nomatch=NA),'age_cd4_cd8_ratio_at_first_visit']



vars2chk <- Cs(hba1cGT_overall,
               dm_dx_overall,
               diabetes_event_overall,
               smoking_ever,
               dyslipidemia_dx_overall,
               dyslipidemia_event_overall,
               ldlGT_overall,
               nafld_dx_overall,
               nafld_event_overall,
               mi_dx_overall,
               cad_dx_overall,
               pvd_dx_overall,
               tia_dx_overall,
               cvd_dx_overall,
               htn_dx_overall,
               hf_dx_overall,
               cvd_event_overall,
               cvd_or_thrombosis_event_overall,
               metabolic_disorder_event_overall,
               hcv_status_ever,
               obese_dx_overall,
               thrombosis_event_overall)
vars2chkPrior <- grep('prior_to',names(df),value=TRUE)
vars2chk <- c(vars2chk,
              vars2chkPrior)
# For those who did not have the conditions specified in vars2chk, setting the indicators to 0 since they would be missing when the
# value was matched in from the other data:
df[,vars2chk] <- apply(df[,vars2chk], 2, FUN=function(y){
                    y <- ifelse(is.na(y),0,y)
                 })
df <- df[order(df$cfar_pid, df$age),]


if(FALSE){
# Merging in data from pregnancies data:
pregnancies_wide <- as.data.frame(pregnancies_wide)
df <- merge(df, pregnancies_wide, all.x=TRUE)
df <- df[order(df$cfar_pid, df$age),]

# Creating indicator for whether time was between conception and delivery of a pregnancy:
df$during_pregnancy <- apply(df[,c('age','age_at_conception.1','age_at_delivery.1',
                                         'age_at_conception.2','age_at_delivery.2',
                                         'age_at_conception.3','age_at_delivery.3',
                                         'age_at_conception.4','age_at_delivery.4')], 1, FUN=function(y){
                                y1 <- as.numeric(!is.na(y[['age_at_conception.1']]) &
                                                 !is.na(y[['age_at_delivery.1']]) &
                                                 y[['age']] >= y[['age_at_conception.1']] &
                                                 y[['age']] <= y[['age_at_delivery.1']])
                                y2 <- as.numeric(!is.na(y[['age_at_conception.2']]) &
                                                 !is.na(y[['age_at_delivery.2']]) &
                                                 y[['age']] >= y[['age_at_conception.2']] &
                                                 y[['age']] <= y[['age_at_delivery.2']])
                                y3 <- as.numeric(!is.na(y[['age_at_conception.3']]) &
                                                 !is.na(y[['age_at_delivery.3']]) &
                                                 y[['age']] >= y[['age_at_conception.3']] &
                                                 y[['age']] <= y[['age_at_delivery.3']])
                                y4 <- as.numeric(!is.na(y[['age_at_conception.4']]) &
                                                 !is.na(y[['age_at_delivery.4']]) &
                                                 y[['age']] >= y[['age_at_conception.4']] &
                                                 y[['age']] <= y[['age_at_delivery.4']])
                                yO <- ifelse(y1==1,1,
                                             ifelse(y2 == 1,2,
                                                    ifelse(y3 ==  1,3,
                                                           ifelse(y4 == 1, 4, NA))))
                                #as.numeric(y1 | y2 | y3 | y4)
                                return(yO)
                                         })
df$during_pregnancy <- ifelse(is.na(df$during_pregnancy),0,df$during_pregnancy)
df$during_pregnancy <- ifelse(!is.na(df$new_baseline_age) & df$age == df$new_baseline_age,0,df$during_pregnancy)
df$during_pregnancy_next <- NA
split(df$during_pregnancy_next, df$cfar_pid) <- lapply(split(df$during_pregnancy, df$cfar_pid), FUN=function(y){
                                                      c(y[-1],0)
                                                })
#df$max_age_followup <- NA
#split(df$max_age_followup, df$cfar_pid) <- lapply(split(df$age, df$cfar_pid), FUN=max)
#df$max_to_censor <- df$age_at_censor - df$max_age_followup
}


df$min_age_followup <- NA
split(df$min_age_followup, df$cfar_pid) <- lapply(split(df$age, df$cfar_pid), FUN=min)

# There are 4 IDs whose max_age_followup is before the age_at_conception.1/age_at_delivery.1 so will change those ages of conception/delivery to missing:
df$age_at_delivery.1 <- ifelse(!is.na(df$age_at_delivery.1) & df$max_age_followup < df$age_at_conception.1,NA,df$age_at_delivery.1)
df$age_at_conception.1 <- ifelse(!is.na(df$age_at_conception.1) & df$max_age_followup < df$age_at_conception.1,NA,df$age_at_conception.1)

# There are 3 IDs whose min_age_followup is before the age_at_delivery.1 so will change those ages of conception/delivery to missing:
condMin <- !is.na(df$age_at_conception.1) & df$min_age_followup > df$age_at_delivery.1
df$age_at_delivery.1 <- ifelse(!is.na(df$age_at_conception.1) & df$min_age_followup > df$age_at_delivery.1,NA,df$age_at_delivery.1)
df$age_at_conception.1 <- ifelse(!is.na(df$age_at_conception.1) & df$min_age_followup > df$age_at_delivery.1,NA,df$age_at_conception.1)


#####
# For calculating exposure time, I guess for those that were pregnant, I need to readjust the monthly data frame to start up when
# they deliver rather than just keep months incrementing as originally designed???
df$age_pregnancy_tmp <- ifelse(df$during_pregnancy == 0 & df$during_pregnancy_next == 1, df$age_at_conception.1,
                           ifelse(df$during_pregnancy == 0 & df$during_pregnancy_next == 2, df$age_at_conception.2,
                           ifelse(df$during_pregnancy == 0 & df$during_pregnancy_next == 3, df$age_at_conception.3,
                           ifelse(df$during_pregnancy == 0 & df$during_pregnancy_next == 4, df$age_at_conception.4,
                           ifelse(df$during_pregnancy == 1 & df$during_pregnancy_next == 0, df$age_at_delivery.1,
                           ifelse(df$during_pregnancy == 2 & df$during_pregnancy_next == 0, df$age_at_delivery.2,
                           ifelse(df$during_pregnancy == 3 & df$during_pregnancy_next == 0, df$age_at_delivery.3,
                           ifelse(df$during_pregnancy == 4 & df$during_pregnancy_next == 0, df$age_at_delivery.4,
                        NA))))))))
df$for_cum_age_pregnancy_1 <- ifelse(is.na(df$age_pregnancy_tmp),df$age,
                                 ifelse(!is.na(df$age_pregnancy_tmp) & df$during_pregnancy== 0 & df$during_pregnancy_next > 0,df$age,
                                 ifelse(!is.na(df$age_pregnancy_tmp) & df$during_pregnancy > 0 & df$during_pregnancy_next == 0,
                                 df$age_pregnancy_tmp,NA)))
df$age_pregnancy_next <- NA
split(df$age_pregnancy_next, df$cfar_pid) <- lapply(split(df$age, df$cfar_pid), FUN=function(y){
                                                c(y[-1],NA)
                                             })
df$for_cum_age_pregnancy_2 <- ifelse(is.na(df$age_pregnancy_tmp), df$age_pregnancy_next,
                                 ifelse(!is.na(df$age_pregnancy_tmp) & df$during_pregnancy == 0 & df$during_pregnancy_next > 0, df$age_pregnancy_tmp,
                                 ifelse(!is.na(df$age_pregnancy_tmp) & df$during_pregnancy > 0 & df$during_pregnancy_next == 0, df$age_pregnancy_next, NA)))

df$cumAgePregnancyDiff <- ifelse(df$during_pregnancy > 0 & df$during_pregnancy_next > 0,0,df$for_cum_age_pregnancy_2 - df$for_cum_age_pregnancy_1)

# Added 3/26/2022:
# Now setting cumAgePregnancyDiff to 0 when cd4_or_vl_12month_gap == 1
df$cumAgePregnancyDiff[which(df$cd4_or_vl_12month_gap == 1)] <- 0
#################################################

df$cumYrsPregnancy <- NA
split(df$cumYrsPregnancy, df$cfar_pid) <- lapply(split(df$cumAgePregnancyDiff, df$cfar_pid), FUN=function(y){
                                             cumsum(y)
                                          })
df$cumYrsPregnancyMax <- NA
split(df$cumYrsPregnancyMax, df$cfar_pid) <- lapply(split(df$cumYrsPregnancy, df$cfar_pid), FUN=function(y){
                                                if(all(is.na(y))){
                                                  tmp <- 0
                                                } else {
                                                   tmp <- max(y, na.rm=TRUE)
                                                } })

# -------------------------------------- #
#       CVD Events during pregnancy      #
# -------------------------------------- #
eventDuringPregnancyCheck <- function(d, evtInd, ageVar,pregAge1='for_cum_age_pregnancy_1',
                                      cumAge1,cumAge2,newCumAge2,evtIndDuringPregnancyNme,cumAgeDiff,cumYrs){
   d[[evtIndDuringPregnancyNme]] <- ifelse(d[[evtInd]] == 0,0,
                                           ifelse(d[[evtInd]] == 1 &
                                                  d$during_pregnancy > 0 & d$during_pregnancy_next > 0 &
                                                  d[[ageVar]] >= d[[cumAge1]] &
                                                  d[[ageVar]] <= d[[cumAge2]],1,NA))

   d[[newCumAge2]] <- ifelse(d[[evtIndDuringPregnancyNme]] %in% c(NA,0),
                             d[[cumAge2]],
#                             ifelse(d[[evtIndDuringPregnancyNme]] == 1, d[[ageVar]],-98))
                             ifelse(d[[evtIndDuringPregnancyNme]] == 1, d[[pregAge1]],-98))

   d[[newCumAge2]] <- ifelse(!is.na(d[[ageVar]]) & d$age > d[[ageVar]],NA,d[[newCumAge2]])

   d[[cumAgeDiff]] <- ifelse(d$during_pregnancy > 0 & d$during_pregnancy_next > 0,0,
                             d[[newCumAge2]] - d[[cumAge1]])
   d[[cumYrs]] <- NA
   split(d[[cumYrs]], d$cfar_pid) <- lapply(split(d[[cumAgeDiff]], d$cfar_pid), FUN=function(y){
                                        cumsum(y)
                                     })
   d[[cumYrs]] <- ifelse(is.na(d[[newCumAge2]]),NA,d[[cumYrs]])

   return(d)

}

df <- eventDuringPregnancyCheck(d = df,
                                evtInd = 'cvd_event_overall',
                                ageVar = 'cvd_event_overall_age',
                                cumAge1 = 'for_cum_age_pregnancy_1',
                                cumAge2 = 'for_cum_age_pregnancy_2',
                                evtIndDuringPregnancyNme = 'cvd_event_during_pregnancy',
                                newCumAge2 = 'for_cum_age_cvd_event_2',
                                cumAgeDiff = 'cumAgeCVDDiff',
                                cumYrs = 'cumYrsCVD')
## Changed 3/1/2022:
#df$cumYrsCVDMax <- NA
#split(df$cumYrsCVDMax, df$cfar_pid) <- lapply(split(df$cumYrsCVD, df$cfar_pid), FUN=function(y){
#                                          max(y, na.rm=TRUE)})
df$cumYrsCVDMax <- NA
split(df$cumYrsCVDMax, df$cfar_pid) <- lapply(split(df$cumYrsCVD, df$cfar_pid), FUN=function(y){
                                          if(all(is.na(y))){
                                            tmp <- 0
                                          } else {
                                            tmp <- max(y, na.rm=TRUE)
                                          }
                                          })

#####
df <- eventDuringPregnancyCheck(d = df,
                                evtInd = 'cvd_or_thrombosis_event_overall',
                                ageVar = 'cvd_or_thrombosis_event_overall_age',
                                cumAge1 = 'for_cum_age_pregnancy_1',
                                cumAge2 = 'for_cum_age_pregnancy_2',
                                evtIndDuringPregnancyNme = 'cvd_or_thrombosis_event_during_pregnancy',
                                newCumAge2 = 'for_cum_age_cvd_or_thrombosis_event_2',
                                cumAgeDiff = 'cumAgeCVDThrombosisDiff',
                                cumYrs = 'cumYrsCVDThrombosis')
## Changed 3/1/2022:
#df$cumYrsCVDThrombosisMax <- NA
#split(df$cumYrsCVDThrombosisMax, df$cfar_pid) <- lapply(split(df$cumYrsCVDThrombosis, df$cfar_pid), FUN=function(y){
#                                                    max(y, na.rm=TRUE)})

df$cumYrsCVDThrombosisMax <- NA
split(df$cumYrsCVDThrombosisMax, df$cfar_pid) <- lapply(split(df$cumYrsCVDThrombosis, df$cfar_pid), FUN=function(y){
                                                    if(all(is.na(y))){
                                                      tmp <- 0
                                                    } else {
                                                      tmp <- max(y, na.rm=TRUE)
                                                    }
                                                    })
######

df <- eventDuringPregnancyCheck(d = df,
                                evtInd = 'metabolic_disorder_event_overall',
                                ageVar = 'metabolic_disorder_event_overall_age',
                                cumAge1 = 'for_cum_age_pregnancy_1',
                                cumAge2 = 'for_cum_age_pregnancy_2',
                                evtIndDuringPregnancyNme = 'metabolic_disorder_event_during_pregnancy',
                                newCumAge2 = 'for_cum_age_metabolic_disorder_event_2',
                                cumAgeDiff = 'cumAgeMetabolicDisorderDiff',
                                cumYrs = 'cumYrsMetabolicDisorder')
## Changed 3/1/2022:
#df$cumYrsMetabolicDisorderMax <- NA
#split(df$cumYrsMetabolicDisorderMax, df$cfar_pid) <- lapply(split(df$cumYrsMetabolicDisorder, df$cfar_pid), FUN=function(y){
#                                                        max(y, na.rm=TRUE)})
df$cumYrsMetabolicDisorderMax <- NA
split(df$cumYrsMetabolicDisorderMax, df$cfar_pid) <- lapply(split(df$cumYrsMetabolicDisorder, df$cfar_pid), FUN=function(y){
                                                        if(all(is.na(y))){
                                                           tmp <- 0
                                                        } else {
                                                           tmp <- max(y, na.rm=TRUE)
                                                        }
                                                        })

######

#df$for_cum_age_cvd_event_2 <- ifelse(df$cvd_event_during_pregnancy %in% c(NA,0),df$for_cum_age_pregnancy_2,
#                                     ifelse(df$cvd_event_during_pregnancy == 1, df$cvd_event_overall_age,-98))
#df$for_cum_age_cvd_event_2 <- ifelse(!is.na(df$cvd_event_overall_age) & df$age > df$cvd_event_overall_age,NA,df$for_cum_age_cvd_event_2)

#df$cumAgeCVDDiff <- ifelse(df$during_pregnancy > 0 & df$during_pregnancy_next > 0,0,df$for_cum_age_cvd_event_2 - df$for_cum_age_pregnancy_1)
#df$cumYrsCVD <- NA
#split(df$cumYrsCVD, df$cfar_pid) <- lapply(split(df$cumAgeCVDDiff, df$cfar_pid), FUN=function(y){
#                                       cumsum(y)
#                                    })



# ------------------------------------------------------------------ #
# Merging in data from hormonal contraceptive data:
hc_wide <- as.data.frame(hc_wide)
df <- merge(df, hc_wide, all.x=TRUE)
df <- df[order(df$cfar_pid, df$age),]
## Added 3/1/2022
df$iud.1[which(is.na(df$iud.1))] <- 0
df$iud.2[which(is.na(df$iud.2))] <- 0
df$iud.3[which(is.na(df$iud.3))] <- 0
df$iud.4[which(is.na(df$iud.4))] <- 0
df$iud.5[which(is.na(df$iud.5))] <- 0
df$iud.6[which(is.na(df$iud.6))] <- 0
df$iud.7[which(is.na(df$iud.7))] <- 0
df$depo.1[which(is.na(df$depo.1))] <- 0
df$depo.2[which(is.na(df$depo.2))] <- 0
df$depo.3[which(is.na(df$depo.3))] <- 0
df$depo.4[which(is.na(df$depo.4))] <- 0
df$depo.5[which(is.na(df$depo.5))] <- 0
df$depo.6[which(is.na(df$depo.6))] <- 0
df$depo.7[which(is.na(df$depo.7))] <- 0
#########


df$during_hc_use_any <- apply(df[,c('age',
                                    'age_at_med_start_updated.1','age_at_med_stop_updated.1','iud.1',
                                    'age_at_med_start_updated.2','age_at_med_stop_updated.2','iud.2',
                                    'age_at_med_start_updated.3','age_at_med_stop_updated.3','iud.3',
                                    'age_at_med_start_updated.4','age_at_med_stop_updated.4','iud.4',
                                    'age_at_med_start_updated.5','age_at_med_stop_updated.5','iud.5',
                                    'age_at_med_start_updated.6','age_at_med_stop_updated.6','iud.6',
                                    'age_at_med_start_updated.7','age_at_med_stop_updated.7','iud.7')], 1, FUN=function(y){
                                    y1 <- as.numeric(!is.na(y[['age_at_med_start_updated.1']]) &
                                                     !is.na(y[['age_at_med_stop_updated.1']]) & y[['iud.1']] == 0 &
                                                     y[['age']] >= y[['age_at_med_start_updated.1']] &
                                                     y[['age']] <= y[['age_at_med_stop_updated.1']])
                                    y2 <- as.numeric(!is.na(y[['age_at_med_start_updated.2']]) &
                                                     !is.na(y[['age_at_med_stop_updated.2']]) & y[['iud.2']] == 0 &
                                                     y[['age']] >= y[['age_at_med_start_updated.2']] &
                                                     y[['age']] <= y[['age_at_med_stop_updated.2']])
                                    y3 <- as.numeric(!is.na(y[['age_at_med_start_updated.3']]) &
                                                     !is.na(y[['age_at_med_stop_updated.3']]) & y[['iud.3']] == 0 &
                                                     y[['age']] >= y[['age_at_med_start_updated.3']] &
                                                     y[['age']] <= y[['age_at_med_stop_updated.3']])
                                    y4 <- as.numeric(!is.na(y[['age_at_med_start_updated.4']]) &
                                                     !is.na(y[['age_at_med_stop_updated.4']]) & y[['iud.4']] == 0 &
                                                     y[['age']] >= y[['age_at_med_start_updated.4']] &
                                                     y[['age']] <= y[['age_at_med_stop_updated.4']])
                                    y5 <- as.numeric(!is.na(y[['age_at_med_start_updated.5']]) &
                                                     !is.na(y[['age_at_med_stop_updated.5']]) & y[['iud.5']] == 0 &
                                                     y[['age']] >= y[['age_at_med_start_updated.5']] &
                                                     y[['age']] <= y[['age_at_med_stop_updated.5']])
                                    y6 <- as.numeric(!is.na(y[['age_at_med_start_updated.6']]) &
                                                     !is.na(y[['age_at_med_stop_updated.6']]) & y[['iud.6']] == 0 &
                                                     y[['age']] >= y[['age_at_med_start_updated.6']] &
                                                     y[['age']] <= y[['age_at_med_stop_updated.6']])
                                    y7 <- as.numeric(!is.na(y[['age_at_med_start_updated.7']]) &
                                                     !is.na(y[['age_at_med_stop_updated.7']]) & y[['iud.7']] == 0 &
                                                     y[['age']] >= y[['age_at_med_start_updated.7']] &
                                                     y[['age']] <= y[['age_at_med_stop_updated.7']])
                                    yO <- ifelse(y1==1,1,
                                                 ifelse(y2 == 1,2,
                                                        ifelse(y3 ==  1,3,
                                                               ifelse(y4 == 1, 4,
                                                               ifelse(y5 == 1, 5,
                                                               ifelse(y6 == 1, 6,
                                                               ifelse(y7 == 1, 7, NA)))))))
                                    return(yO)
                                         })
df$during_hc_use_any <- ifelse(is.na(df$during_hc_use_any),0,df$during_hc_use_any)
df$during_hc_use_any[which(df$cfar_pid == '981062K' & df$month == 4)] <- 2
df$during_hc_use_any[which(df$cfar_pid == '981062K' & df$age == 26.74060)] <- 0
df$during_hc_use_any[which(df$cfar_pid == '996551I' & df$age == 32.7803)] <- 0
df$during_hc_use_any_next <- NA
split(df$during_hc_use_any_next, df$cfar_pid) <- lapply(split(df$during_hc_use_any, df$cfar_pid), FUN=function(y){
                                                          c(y[-1],0)
                                                  })

# For those whose records indicate they were both on hormonal contraception and pregnant, will turn off the hormonal contraception indicator:
df$during_hc_use_any <- ifelse(df$during_hc_use_any > 0 & df$during_pregnancy > 0, 0, df$during_hc_use_any)

# Added 3/26/2022:
# If records indicate both on hormonal contraception and during a lab gap, will turn off the hormonal contraception indicator:
df$during_hc_use_any <- ifelse(df$during_hc_use_any > 0 & df$cd4_or_vl_12month_gap == 1,0,df$during_hc_use_any)

# ???How to adjust for during_hc_use_any_next or do I need to???
df$during_hc_use_any_next <- NA
split(df$during_hc_use_any_next, df$cfar_pid) <- lapply(split(df$during_hc_use_any, df$cfar_pid), FUN=function(y){
                                                          c(y[-1],0)
                                                  })

#################################################
df$during_hc_use_any_next <- ifelse(df$during_hc_use_any_next > 0 & df$during_pregnancy_next > 0,0,df$during_hc_use_any_next)

# --------------------------------------------------- #
df$age_hc_use_any_tmp <- ifelse(df$during_hc_use_any == 0 & df$during_hc_use_any_next == 1, df$age_at_med_start_updated.1,
                           ifelse(df$during_hc_use_any == 0 & df$during_hc_use_any_next == 2, df$age_at_med_start_updated.2,
                           ifelse(df$during_hc_use_any == 0 & df$during_hc_use_any_next == 3, df$age_at_med_start_updated.3,
                           ifelse(df$during_hc_use_any == 0 & df$during_hc_use_any_next == 4, df$age_at_med_start_updated.4,
                           ifelse(df$during_hc_use_any == 0 & df$during_hc_use_any_next == 5, df$age_at_med_start_updated.5,
                           ifelse(df$during_hc_use_any == 0 & df$during_hc_use_any_next == 6, df$age_at_med_start_updated.6,
                           ifelse(df$during_hc_use_any == 0 & df$during_hc_use_any_next == 7, df$age_at_med_start_updated.7,
                           ifelse(df$during_hc_use_any == 1 & df$during_hc_use_any_next == 0, df$age_at_med_stop_updated.1,
                           ifelse(df$during_hc_use_any == 2 & df$during_hc_use_any_next == 0, df$age_at_med_stop_updated.2,
                           ifelse(df$during_hc_use_any == 3 & df$during_hc_use_any_next == 0, df$age_at_med_stop_updated.3,
                           ifelse(df$during_hc_use_any == 4 & df$during_hc_use_any_next == 0, df$age_at_med_stop_updated.4,
                           ifelse(df$during_hc_use_any == 5 & df$during_hc_use_any_next == 0, df$age_at_med_stop_updated.5,
                           ifelse(df$during_hc_use_any == 6 & df$during_hc_use_any_next == 0, df$age_at_med_stop_updated.6,
                           ifelse(df$during_hc_use_any == 7 & df$during_hc_use_any_next == 0, df$age_at_med_stop_updated.7,
                        NA))))))))))))))
df$for_cum_age_hc_use_any_1 <- ifelse(!is.na(df$age_hc_use_any_tmp) & df$during_hc_use_any == 0 & df$during_hc_use_any_next > 0,df$age_hc_use_any_tmp,
                                 ifelse(df$during_hc_use_any > 0 & df$during_hc_use_any_next > 0, df$age,
                                 ifelse(!is.na(df$age_hc_use_any_tmp) & df$during_hc_use_any > 0 & df$during_hc_use_any_next == 0,df$age,NA)))

## Added 4/12/2022
# One record where the for_cum_age_hc_use_any_1 is > current age + 28/365.25
# Will set it to NA
condGT1Month <- !is.na(df$for_cum_age_hc_use_any_1) & df$age + 28/365.25 < df$for_cum_age_hc_use_any_1
df$for_cum_age_hc_use_any_1[condGT1Month] <- NA
######################


df$age_hc_use_any_next <- NA
split(df$age_hc_use_any_next, df$cfar_pid) <- lapply(split(df$age, df$cfar_pid), FUN=function(y){
                                                 c(y[-1],NA)
                                              })
df$for_cum_age_hc_use_any_2 <- ifelse(!is.na(df$age_hc_use_any_tmp) & df$during_hc_use_any == 0 & df$during_hc_use_any_next > 0,
                                      df$age_hc_use_any_next,
                                 ifelse(df$during_hc_use_any > 0 & df$during_hc_use_any_next > 0, df$age_hc_use_any_next,
                                 ifelse(!is.na(df$age_hc_use_any_tmp) & df$during_hc_use_any > 0 & df$during_hc_use_any_next == 0,
                                    df$age_hc_use_any_tmp,
                                        NA)))

### Added 4/12/2022
# Setting the 1 from condGT1Month to have a missing value for for_cum_age_hc_use_any_2
df$for_cum_age_hc_use_any_2[condGT1Month] <- NA
###########################
# Adjusting 2nd age to be age at conception if HC use and pregnancy overlap:
df$for_cum_age_hc_use_any_2 <- ifelse(df$during_pregnancy == 0 & df$during_pregnancy_next > 0 &
                                      !is.na(df$for_cum_age_hc_use_any_2) &
                                      df$for_cum_age_hc_use_any_2 > df$for_cum_age_pregnancy_2,
                                       df$for_cum_age_pregnancy_2,df$for_cum_age_hc_use_any_2)
# Adjusting 2nd age when end of pregnancy and end of HC use are on the same record:
df$for_cum_age_hc_use_any_2 <- apply(df[,c('during_pregnancy','during_pregnancy_next',
                                            'during_hc_use_any','during_hc_use_any_next',
                                            'age_at_delivery.1','age_at_med_stop_updated.1',
                                            'age_at_delivery.3','age_at_med_stop_updated.3',
                                            'for_cum_age_hc_use_any_2')], 1, FUN=function(y){
                                   tmp <- ifelse(y[['during_pregnancy']] == 1 & y[['during_pregnancy_next']] == 0 &
                                                 y[['during_hc_use_any']] == 1 & y[['during_hc_use_any_next']] == 0,
                                                 max(c(as.numeric(y[['age_at_delivery.1']]), as.numeric(y[['age_at_med_stop_updated.1']])),na.rm=TRUE),
                                                 ifelse(y[['during_pregnancy']] == 3 & y[['during_pregnancy_next']] == 0 &
                                                        y[['during_hc_use_any']] == 1 & y[['during_hc_use_any_next']] == 0,
                                                        max(c(as.numeric(y[['age_at_delivery.3']]),as.numeric(y[['age_at_med_stop_updated.1']])),na.rm=TRUE),
                                                        ifelse(y[['during_pregnancy']] == 3 & y[['during_pregnancy_next']] == 0 &
                                                               y[['during_hc_use_any']] == 3 & y[['during_hc_use_any_next']] == 3,
                                                               max(c(as.numeric(y[['age_at_delivery.3']]), as.numeric(y[['age_at_med_stop_updated.3']])),na.rm=TRUE),
                                                               y[['for_cum_age_hc_use_any_2']])))
                                   return(tmp) })


df$for_cum_age_hc_use_any_1 <- ifelse(df$during_pregnancy > 0 & df$during_pregnancy_next == 0 &
                                      !is.na(df$for_cum_age_hc_use_any_1) &
                                      df$for_cum_age_hc_use_any_1 < df$for_cum_age_pregnancy_1,
                                      df$for_cum_age_pregnancy_1,df$for_cum_age_hc_use_any_1)

# Need to adjust if they switch from one HC to another but the end date != the next start date:
cond1t2 <- which(df$during_hc_use_any == 1 & df$during_hc_use_any_next == 2)
df$for_cum_age_hc_use_any_2[cond1t2] <- df$age_at_med_stop_updated.1[cond1t2]
df$for_cum_age_hc_use_any_1[cond1t2+1] <- df$age_at_med_start_updated.2[cond1t2+1]

cond1t3 <- which(df$during_hc_use_any == 1 & df$during_hc_use_any_next == 3)
df$for_cum_age_hc_use_any_2[cond1t3] <- df$age_at_med_stop_updated.1[cond1t3]
df$for_cum_age_hc_use_any_1[cond1t3+1] <- df$age_at_med_start_updated.3[cond1t3+1]

cond2t3 <- which(df$during_hc_use_any == 2 & df$during_hc_use_any_next == 3)
df$for_cum_age_hc_use_any_2[cond2t3] <- df$age_at_med_stop_updated.2[cond2t3]
df$for_cum_age_hc_use_any_1[cond2t3+1] <- df$age_at_med_start_updated.3[cond2t3+1]

cond3t4 <- which(df$during_hc_use_any == 3 & df$during_hc_use_any_next == 4)
df$for_cum_age_hc_use_any_2[cond3t4] <- df$age_at_med_stop_updated.3[cond3t4]
df$for_cum_age_hc_use_any_1[cond3t4+1] <- df$age_at_med_start_updated.4[cond3t4+1]


cond4t5 <- which(df$during_hc_use_any == 4 & df$during_hc_use_any_next == 5)
df$for_cum_age_hc_use_any_2[cond4t5] <- df$age_at_med_stop_updated.4[cond4t5]
df$for_cum_age_hc_use_any_1[cond4t5+1] <- df$age_at_med_start_updated.5[cond4t5+1]



df$cumAgeHCUseAnyDiff <- ifelse(df$during_hc_use_any == 0 & df$during_hc_use_any_next > 0,df$for_cum_age_hc_use_any_2 - df$for_cum_age_hc_use_any_1,
                                ifelse(df$during_hc_use_any > 0 & df$during_hc_use_any_next == 0,df$for_cum_age_hc_use_any_2 - df$for_cum_age_hc_use_any_1,
                                ifelse(df$during_hc_use_any > 0 & df$during_hc_use_any_next > 0,df$for_cum_age_hc_use_any_2 - df$for_cum_age_hc_use_any_1,
                                ifelse(df$during_hc_use_any == 0 & df$during_hc_use_any_next == 0,0,NA))))

# Added 4/8/2022:
df$for_cum_age_hc_use_any_1 <- ifelse(df$cd4_or_vl_12month_gap == 1,NA,df$for_cum_age_hc_use_any_1)
df$for_cum_age_hc_use_any_2 <- ifelse(df$cd4_or_vl_12month_gap == 1,NA,df$for_cum_age_hc_use_any_2)
df$cd4_or_vl_12month_gap_next <- NA
split(df$cd4_or_vl_12month_gap_next, df$cfar_pid) <- lapply(split(df$cd4_or_vl_12month_gap,df$cfar_pid), FUN=function(y){
                                                        if(length(y) > 1){
                                                          tmp <- c(y[-1],NA)
                                                        } else {
                                                          tmp <- NA
                                                        }
                                                        return(tmp)
                                                        })
df$age_next <- NA
split(df$age_next, df$cfar_pid) <- lapply(split(df$age,df$cfar_pid), FUN=function(y){
                                                        if(length(y) > 1){
                                                          tmp <- c(y[-1],NA)
                                                        } else {
                                                          tmp <- NA
                                                        }
                                                        return(tmp)
                                                        })

df$for_cum_age_hc_use_any_2 <- ifelse(!is.na(df$cd4_or_vl_12month_gap_next) & df$cd4_or_vl_12month_gap == 0 &
                                       df$cd4_or_vl_12month_gap_next == 1,
                                       df$age_next, df$for_cum_age_hc_use_any_2)
df$cumAgeHCUseAnyDiff <- ifelse(df$cd4_or_vl_12month_gap == 1,0,df$cumAgeHCUseAnyDiff)
df$cumAgeHCUseAnyDiff <- ifelse(!is.na(df$cd4_or_vl_12month_gap_next) & df$cd4_or_vl_12month_gap == 0 &
                                !is.na(df$cd4_or_vl_12month_gap) & df$cd4_or_vl_12month_gap_next == 1,
                                df$for_cum_age_hc_use_any_2 - df$for_cum_age_hc_use_any_1,df$cumAgeHCUseAnyDiff)
df$cumAgeHCUseAnyDiff[which(is.na(df$cumAgeHCUseAnyDiff))] <- 0                                
##################################

df$cumYrsHCUseAny <- NA
split(df$cumYrsHCUseAny, df$cfar_pid) <- lapply(split(df$cumAgeHCUseAnyDiff, df$cfar_pid), FUN=function(y){
                                             cumsum(y)
                                          })

df$cumYrsHCUseAnyMax <- NA
split(df$cumYrsHCUseAnyMax, df$cfar_pid) <- lapply(split(df$cumYrsHCUseAny, df$cfar_pid), FUN=function(y){
                                               max(y, na.rm=TRUE) })


df$hc_use_any <- as.numeric(df$cumYrsHCUseAnyMax > 0)
df$hc_use_any.factor <- factor(df$hc_use_any, levels=c(0,1), labels=c('No HC Use','HC Use'))

# And creating an indicator (0/1) for on/off HC (time-varying):
df$hc_use_any_indicator <- ifelse(df$during_hc_use_any > 0,1,0)
df$hc_use_any_indicator.factor <- factor(df$hc_use_any_indicator, levels=c(0,1), labels=c('No','Yes'))
label(df$hc_use_any_indicator.factor) <- 'HC use indicator (any type; time-varying)'

# ------------------------------------------- #
#         Cumulative time on oral HC          #
# ------------------------------------------- #
df <- df[order(df$cfar_pid, df$age),]

df$during_hc_use_oral <- apply(df[,c('age',
                                    'age_at_med_start_updated.1','age_at_med_stop_updated.1','depo.1','iud.1',
                                    'age_at_med_start_updated.2','age_at_med_stop_updated.2','depo.2','iud.2',
                                    'age_at_med_start_updated.3','age_at_med_stop_updated.3','depo.3','iud.3',
                                    'age_at_med_start_updated.4','age_at_med_stop_updated.4','depo.4','iud.4',
                                    'age_at_med_start_updated.5','age_at_med_stop_updated.5','depo.5','iud.5',
                                    'age_at_med_start_updated.6','age_at_med_stop_updated.6','depo.6','iud.6',
                                    'age_at_med_start_updated.7','age_at_med_stop_updated.7','depo.7','iud.7')], 1, FUN=function(y){
                                    y1 <- as.numeric(!is.na(y[['age_at_med_start_updated.1']]) &
                                                     !is.na(y[['age_at_med_stop_updated.1']]) &
                                                     y[['depo.1']] == 0 & y[['iud.1']] == 0 &
                                                     y[['age']] >= y[['age_at_med_start_updated.1']] &
                                                     y[['age']] <= y[['age_at_med_stop_updated.1']])
                                    y2 <- as.numeric(!is.na(y[['age_at_med_start_updated.2']]) &
                                                     !is.na(y[['age_at_med_stop_updated.2']]) &
                                                     y[['depo.2']] == 0 & y[['iud.2']] == 0 &
                                                     y[['age']] >= y[['age_at_med_start_updated.2']] &
                                                     y[['age']] <= y[['age_at_med_stop_updated.2']])
                                    y3 <- as.numeric(!is.na(y[['age_at_med_start_updated.3']]) &
                                                     !is.na(y[['age_at_med_stop_updated.3']]) &
                                                     y[['depo.3']] == 0  & y[['iud.3']] == 0 &
                                                     y[['age']] >= y[['age_at_med_start_updated.3']] &
                                                     y[['age']] <= y[['age_at_med_stop_updated.3']])
                                    y4 <- as.numeric(!is.na(y[['age_at_med_start_updated.4']]) &
                                                     !is.na(y[['age_at_med_stop_updated.4']]) &
                                                     y[['depo.4']] == 0  & y[['iud.4']] == 0 &
                                                     y[['age']] >= y[['age_at_med_start_updated.4']] &
                                                     y[['age']] <= y[['age_at_med_stop_updated.4']])
                                    y5 <- as.numeric(!is.na(y[['age_at_med_start_updated.5']]) &
                                                     !is.na(y[['age_at_med_stop_updated.5']]) &
                                                     y[['depo.5']] == 0  & y[['iud.5']] == 0 &
                                                     y[['age']] >= y[['age_at_med_start_updated.5']] &
                                                     y[['age']] <= y[['age_at_med_stop_updated.5']])
                                    y6 <- as.numeric(!is.na(y[['age_at_med_start_updated.6']]) &
                                                     !is.na(y[['age_at_med_stop_updated.6']]) &
                                                     y[['depo.6']] == 0  & y[['iud.6']] == 0 &
                                                     y[['age']] >= y[['age_at_med_start_updated.6']] &
                                                     y[['age']] <= y[['age_at_med_stop_updated.6']])
                                    y7 <- as.numeric(!is.na(y[['age_at_med_start_updated.7']]) &
                                                     !is.na(y[['age_at_med_stop_updated.7']]) &
                                                     y[['depo.7']] == 0  & y[['iud.7']] == 0 &
                                                     y[['age']] >= y[['age_at_med_start_updated.7']] &
                                                     y[['age']] <= y[['age_at_med_stop_updated.7']])
                                    yO <- ifelse(y1==1,1,
                                                 ifelse(y2 == 1,2,
                                                        ifelse(y3 ==  1,3,
                                                               ifelse(y4 == 1, 4,
                                                               ifelse(y5 == 1, 5,
                                                               ifelse(y6 == 1, 6,
                                                               ifelse(y7 == 1, 7, NA)))))))
                                    return(yO)
                                         })
df$during_hc_use_oral <- ifelse(is.na(df$during_hc_use_oral),0,df$during_hc_use_oral)
df$during_hc_use_oral[which(df$cfar_pid == '981062K' & df$age == 26.74060)] <- 0
# Added 3/26/2022:
# If records indicate both on hormonal contraception and during a lab gap, will turn off the hormonal contraception indicator:
df$during_hc_use_oral <- ifelse(df$during_hc_use_oral > 0 & df$cd4_or_vl_12month_gap == 1,0,df$during_hc_use_oral)

# ???How to adjust for during_hc_use_any_next or do I need to???
df$during_hc_use_oral_next <- NA
split(df$during_hc_use_oral_next, df$cfar_pid) <- lapply(split(df$during_hc_use_oral, df$cfar_pid), FUN=function(y){
                                                          c(y[-1],0)
                                                  })

#################################################

df$during_hc_use_oral_next <- NA
split(df$during_hc_use_oral_next, df$cfar_pid) <- lapply(split(df$during_hc_use_oral, df$cfar_pid), FUN=function(y){
                                                          c(y[-1],0)
                                                  })


# For those whose records indicate they were both on hormonal contraception and pregnant, will turn off the hormonal contraception indicator:
df$during_hc_use_oral <- ifelse(df$during_hc_use_oral > 0 & df$during_pregnancy > 0, 0, df$during_hc_use_oral)
df$during_hc_use_oral_next <- ifelse(df$during_hc_use_oral_next > 0 & df$during_pregnancy_next > 0,0,df$during_hc_use_oral_next)



df$age_hc_use_oral_tmp <- ifelse(df$during_hc_use_oral == 0 & df$during_hc_use_oral_next == 1, df$age_at_med_start_updated.1,
                           ifelse(df$during_hc_use_oral == 0 & df$during_hc_use_oral_next == 2, df$age_at_med_start_updated.2,
                           ifelse(df$during_hc_use_oral == 0 & df$during_hc_use_oral_next == 3, df$age_at_med_start_updated.3,
                           ifelse(df$during_hc_use_oral == 0 & df$during_hc_use_oral_next == 4, df$age_at_med_start_updated.4,
                           ifelse(df$during_hc_use_oral == 0 & df$during_hc_use_oral_next == 5, df$age_at_med_start_updated.5,
                           ifelse(df$during_hc_use_oral == 0 & df$during_hc_use_oral_next == 6, df$age_at_med_start_updated.6,
                           ifelse(df$during_hc_use_oral == 0 & df$during_hc_use_oral_next == 7, df$age_at_med_start_updated.7,
                           ifelse(df$during_hc_use_oral == 1 & df$during_hc_use_oral_next == 0, df$age_at_med_stop_updated.1,
                           ifelse(df$during_hc_use_oral == 2 & df$during_hc_use_oral_next == 0, df$age_at_med_stop_updated.2,
                           ifelse(df$during_hc_use_oral == 3 & df$during_hc_use_oral_next == 0, df$age_at_med_stop_updated.3,
                           ifelse(df$during_hc_use_oral == 4 & df$during_hc_use_oral_next == 0, df$age_at_med_stop_updated.4,
                           ifelse(df$during_hc_use_oral == 5 & df$during_hc_use_oral_next == 0, df$age_at_med_stop_updated.5,
                           ifelse(df$during_hc_use_oral == 6 & df$during_hc_use_oral_next == 0, df$age_at_med_stop_updated.6,
                           ifelse(df$during_hc_use_oral == 7 & df$during_hc_use_oral_next == 0, df$age_at_med_stop_updated.7,
                           NA))))))))))))))
df$for_cum_age_hc_use_oral_1 <- ifelse(!is.na(df$age_hc_use_oral_tmp) & df$during_hc_use_oral == 0 & df$during_hc_use_oral_next > 0,df$age_hc_use_oral_tmp,
                                 ifelse(df$during_hc_use_oral > 0 & df$during_hc_use_oral_next > 0, df$age,
                                 ifelse(!is.na(df$age_hc_use_oral_tmp) & df$during_hc_use_oral > 0 & df$during_hc_use_oral_next == 0,df$age,NA)))
df$age_hc_use_oral_next <- NA
split(df$age_hc_use_oral_next, df$cfar_pid) <- lapply(split(df$age, df$cfar_pid), FUN=function(y){
                                                 c(y[-1],NA)
                                              })
df$for_cum_age_hc_use_oral_2 <- ifelse(!is.na(df$age_hc_use_oral_tmp) & df$during_hc_use_oral == 0 & df$during_hc_use_oral_next > 0,df$age_hc_use_oral_next,
                                 ifelse(df$during_hc_use_oral > 0 & df$during_hc_use_oral_next > 0, df$age_hc_use_oral_next,
                                 ifelse(!is.na(df$age_hc_use_oral_tmp) & df$during_hc_use_oral > 0 & df$during_hc_use_oral_next == 0,df$age_hc_use_oral_tmp,
                                        NA)))
# Adjusting 2nd age to be age at conception if HC use and pregnancy overlap:
df$for_cum_age_hc_use_oral_2 <- ifelse(df$during_pregnancy == 0 & df$during_pregnancy_next > 0 &
                                      !is.na(df$for_cum_age_hc_use_oral_2) &
                                      df$for_cum_age_hc_use_oral_2 > df$for_cum_age_pregnancy_2,
                                       df$for_cum_age_pregnancy_2,df$for_cum_age_hc_use_oral_2)
# Adjusting 2nd age when end of pregnancy and end of HC use are on the same record:
df$for_cum_age_hc_use_oral_2 <- apply(df[,c('during_pregnancy','during_pregnancy_next',
                                            'during_hc_use_oral','during_hc_use_oral_next',
                                            'age_at_delivery.1','age_at_med_stop_updated.1',
                                            'age_at_delivery.3','age_at_med_stop_updated.3',
                                            'for_cum_age_hc_use_oral_2')], 1, FUN=function(y){
                                   tmp <- ifelse(y[['during_pregnancy']] == 1 & y[['during_pregnancy_next']] == 0 &
                                                 y[['during_hc_use_oral']] == 1 & y[['during_hc_use_oral_next']] == 0,
                                                 max(c(as.numeric(y[['age_at_delivery.1']]), as.numeric(y[['age_at_med_stop_updated.1']])),na.rm=TRUE),
                                                 ifelse(y[['during_pregnancy']] == 3 & y[['during_pregnancy_next']] == 0 &
                                                        y[['during_hc_use_oral']] == 1 & y[['during_hc_use_oral_next']] == 0,
                                                        max(c(as.numeric(y[['age_at_delivery.3']]),as.numeric(y[['age_at_med_stop_updated.1']])),na.rm=TRUE),
                                                        ifelse(y[['during_pregnancy']] == 3 & y[['during_pregnancy_next']] == 0 &
                                                               y[['during_hc_use_oral']] == 3 & y[['during_hc_use_oral_next']] == 3,
                                                               max(c(as.numeric(y[['age_at_delivery.3']]), as.numeric(y[['age_at_med_stop_updated.3']])),na.rm=TRUE),
                                                               y[['for_cum_age_hc_use_oral_2']])))
                                   return(tmp) })


df$for_cum_age_hc_use_oral_1 <- ifelse(df$during_pregnancy > 0 & df$during_pregnancy_next == 0 &
                                      !is.na(df$for_cum_age_hc_use_oral_1) &
                                      df$for_cum_age_hc_use_oral_1 < df$for_cum_age_pregnancy_1,
                                      df$for_cum_age_pregnancy_1,df$for_cum_age_hc_use_oral_1)

# Manually changing 2 records for Oral for ID 999010J:
condTmp <- df$cfar_pid == '999010J' & round(df$age,5) == 26.12735
df$for_cum_age_hc_use_oral_2[condTmp] <- 26.141

condTmp2 <- df$cfar_pid == '999010J' & round(df$age,5) == 26.20401
df$for_cum_age_hc_use_oral_1[condTmp2] <- 26.14650

df$cumAgeHCUseOralDiff <- ifelse(df$during_hc_use_oral == 0 & df$during_hc_use_oral_next > 0,df$for_cum_age_hc_use_oral_2 - df$for_cum_age_hc_use_oral_1,
                                ifelse(df$during_hc_use_oral > 0 & df$during_hc_use_oral_next == 0,df$for_cum_age_hc_use_oral_2 - df$for_cum_age_hc_use_oral_1,
                                ifelse(df$during_hc_use_oral > 0 & df$during_hc_use_oral_next > 0,df$for_cum_age_hc_use_oral_2 - df$for_cum_age_hc_use_oral_1,
                                ifelse(df$during_hc_use_oral == 0 & df$during_hc_use_oral_next == 0,0,NA))))

# Added 4/8/2022:
df$for_cum_age_hc_use_oral_1 <- ifelse(df$cd4_or_vl_12month_gap == 1,NA,df$for_cum_age_hc_use_oral_1)
df$for_cum_age_hc_use_oral_2 <- ifelse(df$cd4_or_vl_12month_gap == 1,NA,df$for_cum_age_hc_use_oral_2)
df$for_cum_age_hc_use_oral_2 <- ifelse(!is.na(df$cd4_or_vl_12month_gap_next) & df$cd4_or_vl_12month_gap == 0 &
                                       df$cd4_or_vl_12month_gap_next == 1, df$age_next, df$for_cum_age_hc_use_oral_2)

df$cumAgeHCUseOralDiff <- ifelse(df$cd4_or_vl_12month_gap == 1,0,df$cumAgeHCUseOralDiff)
df$cumAgeHCUseOralDiff <- ifelse(!is.na(df$cd4_or_vl_12month_gap_next) & df$cd4_or_vl_12month_gap == 0 &
                                !is.na(df$cd4_or_vl_12month_gap) & df$cd4_or_vl_12month_gap_next == 1,
                                df$for_cum_age_hc_use_oral_2 - df$for_cum_age_hc_use_oral_1,df$cumAgeHCUseOralDiff)
df$cumAgeHCUseOralDiff[which(is.na(df$cumAgeHCUseOralDiff))] <- 0
##################################

df$cumYrsHCUseOral <- NA
split(df$cumYrsHCUseOral, df$cfar_pid) <- lapply(split(df$cumAgeHCUseOralDiff, df$cfar_pid), FUN=function(y){
                                             cumsum(y)
                                          })


df$cumYrsHCUseOralMax <- NA
split(df$cumYrsHCUseOralMax, df$cfar_pid) <- lapply(split(df$cumYrsHCUseOral, df$cfar_pid), FUN=function(y){
                                               max(y, na.rm=TRUE) })
df$hc_use_oral <- as.numeric(df$cumYrsHCUseOralMax > 0)
df$hc_use_oral.factor <- factor(df$hc_use_oral, levels=c(0,1), labels=c('No oral HC Use','Oral HC Use'))

# And creating an indicator (0/1) for on/off HC (time-varying):
df$hc_use_oral_indicator <- ifelse(df$during_hc_use_oral > 0,1,0)
df$hc_use_oral_indicator.factor <- factor(df$hc_use_oral_indicator, levels=c(0,1), labels=c('No','Yes'))
label(df$hc_use_oral_indicator.factor) <- 'Oral HC use indicator (time-varying)'

# -------------------------------------- #
#      Cumulative time on Depo HC        #
# -------------------------------------- #
df <- df[order(df$cfar_pid, df$age),]


df$during_hc_use_depo <- apply(df[,c('age',
                                    'age_at_med_start_updated.1','age_at_med_stop_updated.1','depo.1',
                                    'age_at_med_start_updated.2','age_at_med_stop_updated.2','depo.2',
                                    'age_at_med_start_updated.3','age_at_med_stop_updated.3','depo.3',
                                    'age_at_med_start_updated.4','age_at_med_stop_updated.4','depo.4',
                                    'age_at_med_start_updated.5','age_at_med_stop_updated.5','depo.5',
                                    'age_at_med_start_updated.6','age_at_med_stop_updated.6','depo.6',
                                    'age_at_med_start_updated.7','age_at_med_stop_updated.7','depo.7')], 1, FUN=function(y){
                                    y1 <- as.numeric(!is.na(y[['age_at_med_start_updated.1']]) &
                                                     !is.na(y[['age_at_med_stop_updated.1']]) &
                                                     y[['depo.1']] == 1 &
                                                     y[['age']] >= y[['age_at_med_start_updated.1']] &
                                                     y[['age']] <= y[['age_at_med_stop_updated.1']])
                                    y2 <- as.numeric(!is.na(y[['age_at_med_start_updated.2']]) &
                                                     !is.na(y[['age_at_med_stop_updated.2']]) &
                                                     y[['depo.2']] == 1 &
                                                     y[['age']] >= y[['age_at_med_start_updated.2']] &
                                                     y[['age']] <= y[['age_at_med_stop_updated.2']])
                                    y3 <- as.numeric(!is.na(y[['age_at_med_start_updated.3']]) &
                                                     !is.na(y[['age_at_med_stop_updated.3']]) &
                                                     y[['depo.3']] == 1 &
                                                     y[['age']] >= y[['age_at_med_start_updated.3']] &
                                                     y[['age']] <= y[['age_at_med_stop_updated.3']])
                                    y4 <- as.numeric(!is.na(y[['age_at_med_start_updated.4']]) &
                                                     !is.na(y[['age_at_med_stop_updated.4']]) &
                                                     y[['depo.4']] == 1 &
                                                     y[['age']] >= y[['age_at_med_start_updated.4']] &
                                                     y[['age']] <= y[['age_at_med_stop_updated.4']])
                                    y5 <- as.numeric(!is.na(y[['age_at_med_start_updated.5']]) &
                                                     !is.na(y[['age_at_med_stop_updated.5']]) &
                                                     y[['depo.5']] == 1 &
                                                     y[['age']] >= y[['age_at_med_start_updated.5']] &
                                                     y[['age']] <= y[['age_at_med_stop_updated.5']])
                                    y6 <- as.numeric(!is.na(y[['age_at_med_start_updated.6']]) &
                                                     !is.na(y[['age_at_med_stop_updated.6']]) &
                                                     y[['depo.6']] == 1 &
                                                     y[['age']] >= y[['age_at_med_start_updated.6']] &
                                                     y[['age']] <= y[['age_at_med_stop_updated.6']])
                                    y7 <- as.numeric(!is.na(y[['age_at_med_start_updated.7']]) &
                                                     !is.na(y[['age_at_med_stop_updated.7']]) &
                                                     y[['depo.7']] == 1 &
                                                     y[['age']] >= y[['age_at_med_start_updated.7']] &
                                                     y[['age']] <= y[['age_at_med_stop_updated.7']])
                                    yO <- ifelse(y1==1,1,
                                                 ifelse(y2 == 1,2,
                                                        ifelse(y3 ==  1,3,
                                                               ifelse(y4 == 1, 4,
                                                               ifelse(y5 == 1, 5,
                                                               ifelse(y6 == 1, 6,
                                                               ifelse(y7 == 1, 7, NA)))))))
                                    return(yO)
                                         })
df$during_hc_use_depo <- ifelse(is.na(df$during_hc_use_depo),0,df$during_hc_use_depo)
df$during_hc_use_depo[which(df$cfar_pid == '981062K' & df$month == 4)] <- 2
df$during_hc_use_depo[which(df$cfar_pid == '996551I' & df$age == 32.7803)] <- 0
# Added 3/26/2022:
# If records indicate both on hormonal contraception and during a lab gap, will turn off the hormonal contraception indicator:
df$during_hc_use_depo <- ifelse(df$during_hc_use_depo > 0 & df$cd4_or_vl_12month_gap == 1,0,df$during_hc_use_depo)

# ???How to adjust for during_hc_use_any_next or do I need to???
df$during_hc_use_depo_next <- NA
split(df$during_hc_use_depo_next, df$cfar_pid) <- lapply(split(df$during_hc_use_depo, df$cfar_pid), FUN=function(y){
                                                          c(y[-1],0)
                                                  })

#################################################

df$during_hc_use_depo_next <- NA
split(df$during_hc_use_depo_next, df$cfar_pid) <- lapply(split(df$during_hc_use_depo, df$cfar_pid), FUN=function(y){
                                                          c(y[-1],0)
                                                  })


# For those whose records indicate they were both on hormonal contraception and pregnant, will turn off the hormonal contraception indicator:
df$during_hc_use_depo <- ifelse(df$during_hc_use_depo > 0 & df$during_pregnancy > 0, 0, df$during_hc_use_depo)
df$during_hc_use_depo_next <- ifelse(df$during_hc_use_depo_next > 0 & df$during_pregnancy_next > 0,0,df$during_hc_use_depo_next)

df$age_hc_use_depo_tmp <- ifelse(df$during_hc_use_depo == 0 & df$during_hc_use_depo_next == 1, df$age_at_med_start_updated.1,
                           ifelse(df$during_hc_use_depo == 0 & df$during_hc_use_depo_next == 2, df$age_at_med_start_updated.2,
                           ifelse(df$during_hc_use_depo == 0 & df$during_hc_use_depo_next == 3, df$age_at_med_start_updated.3,
                           ifelse(df$during_hc_use_depo == 0 & df$during_hc_use_depo_next == 4, df$age_at_med_start_updated.4,
                           ifelse(df$during_hc_use_depo == 0 & df$during_hc_use_depo_next == 5, df$age_at_med_start_updated.5,
                           ifelse(df$during_hc_use_depo == 0 & df$during_hc_use_depo_next == 6, df$age_at_med_start_updated.6,
                           ifelse(df$during_hc_use_depo == 0 & df$during_hc_use_depo_next == 7, df$age_at_med_start_updated.7,
                           ifelse(df$during_hc_use_depo == 1 & df$during_hc_use_depo_next == 0, df$age_at_med_stop_updated.1,
                           ifelse(df$during_hc_use_depo == 2 & df$during_hc_use_depo_next == 0, df$age_at_med_stop_updated.2,
                           ifelse(df$during_hc_use_depo == 3 & df$during_hc_use_depo_next == 0, df$age_at_med_stop_updated.3,
                           ifelse(df$during_hc_use_depo == 4 & df$during_hc_use_depo_next == 0, df$age_at_med_stop_updated.4,
                           ifelse(df$during_hc_use_depo == 5 & df$during_hc_use_depo_next == 0, df$age_at_med_stop_updated.5,
                           ifelse(df$during_hc_use_depo == 6 & df$during_hc_use_depo_next == 0, df$age_at_med_stop_updated.6,
                           ifelse(df$during_hc_use_depo == 7 & df$during_hc_use_depo_next == 0, df$age_at_med_stop_updated.7,
                           NA))))))))))))))
df$for_cum_age_hc_use_depo_1 <- ifelse(!is.na(df$age_hc_use_depo_tmp) & df$during_hc_use_depo == 0 & df$during_hc_use_depo_next > 0,df$age_hc_use_depo_tmp,
                                 ifelse(df$during_hc_use_depo > 0 & df$during_hc_use_depo_next > 0, df$age,
                                 ifelse(!is.na(df$age_hc_use_depo_tmp) & df$during_hc_use_depo > 0 & df$during_hc_use_depo_next == 0,df$age,NA)))

## Added 4/12/2022
# One record where the for_cum_age_hc_use_any_1 is > current age + 28/365.25
# Will set it to NA
condGT1MonthDepo <- !is.na(df$for_cum_age_hc_use_depo_1) & df$age + 28/365.25 < df$for_cum_age_hc_use_depo_1
df$for_cum_age_hc_use_depo_1[condGT1MonthDepo] <- NA
######################

df$age_hc_use_depo_next <- NA
split(df$age_hc_use_depo_next, df$cfar_pid) <- lapply(split(df$age, df$cfar_pid), FUN=function(y){
                                                 c(y[-1],NA)
                                              })
df$for_cum_age_hc_use_depo_2 <- ifelse(!is.na(df$age_hc_use_depo_tmp) & df$during_hc_use_depo == 0 & df$during_hc_use_depo_next > 0,df$age_hc_use_depo_next,
                                 ifelse(df$during_hc_use_depo > 0 & df$during_hc_use_depo_next > 0, df$age_hc_use_depo_next,
                                 ifelse(!is.na(df$age_hc_use_depo_tmp) & df$during_hc_use_depo > 0 & df$during_hc_use_depo_next == 0,df$age_hc_use_depo_tmp,
                                        NA)))
## Added 4/12/2022
# One record where the for_cum_age_hc_use_any_1 is > current age + 28/365.25
# Will set it to NA
df$for_cum_age_hc_use_depo_2[condGT1MonthDepo] <- NA
######################

# Adjusting 2nd age to be age at conception if HC use and pregnancy overlap:
df$for_cum_age_hc_use_depo_2 <- ifelse(df$during_pregnancy == 0 & df$during_pregnancy_next > 0 &
                                      !is.na(df$for_cum_age_hc_use_depo_2) &
                                      df$for_cum_age_hc_use_depo_2 > df$for_cum_age_pregnancy_2,
                                       df$for_cum_age_pregnancy_2,df$for_cum_age_hc_use_depo_2)
# Adjusting 2nd age when end of pregnancy and end of HC use are on the same record:
df$for_cum_age_hc_use_depo_2 <- apply(df[,c('during_pregnancy','during_pregnancy_next',
                                            'during_hc_use_depo','during_hc_use_depo_next',
                                            'age_at_delivery.1','age_at_med_stop_updated.1',
                                            'age_at_delivery.3','age_at_med_stop_updated.3',
                                            'for_cum_age_hc_use_depo_2')], 1, FUN=function(y){
                                   tmp <- ifelse(y[['during_pregnancy']] == 1 & y[['during_pregnancy_next']] == 0 &
                                                 y[['during_hc_use_depo']] == 1 & y[['during_hc_use_depo_next']] == 0,
                                                 max(c(as.numeric(y[['age_at_delivery.1']]), as.numeric(y[['age_at_med_stop_updated.1']])),na.rm=TRUE),
                                                 ifelse(y[['during_pregnancy']] == 3 & y[['during_pregnancy_next']] == 0 &
                                                        y[['during_hc_use_depo']] == 1 & y[['during_hc_use_depo_next']] == 0,
                                                        max(c(as.numeric(y[['age_at_delivery.3']]),as.numeric(y[['age_at_med_stop_updated.1']])),na.rm=TRUE),
                                                        ifelse(y[['during_pregnancy']] == 3 & y[['during_pregnancy_next']] == 0 &
                                                               y[['during_hc_use_depo']] == 3 & y[['during_hc_use_depo_next']] == 3,
                                                               max(c(as.numeric(y[['age_at_delivery.3']]), as.numeric(y[['age_at_med_stop_updated.3']])),na.rm=TRUE),
                                                               y[['for_cum_age_hc_use_depo_2']])))
                                   return(tmp) })


df$for_cum_age_hc_use_depo_1 <- ifelse(df$during_pregnancy > 0 & df$during_pregnancy_next == 0 &
                                      !is.na(df$for_cum_age_hc_use_depo_1) &
                                      df$for_cum_age_hc_use_depo_1 < df$for_cum_age_pregnancy_1,
                                      df$for_cum_age_pregnancy_1,df$for_cum_age_hc_use_depo_1)

# Need to adjust if they switch from one HC to another but the end date != the next start date:
cond1t2 <- which(df$during_hc_use_depo == 1 & df$during_hc_use_depo_next == 2)
df$for_cum_age_hc_use_depo_2[cond1t2] <- df$age_at_med_stop_updated.1[cond1t2]
df$for_cum_age_hc_use_depo_1[cond1t2+1] <- df$age_at_med_start_updated.2[cond1t2+1]

cond2t3 <- which(df$during_hc_use_depo == 2 & df$during_hc_use_depo_next == 3)
df$for_cum_age_hc_use_depo_2[cond2t3] <- df$age_at_med_stop_updated.2[cond2t3]
df$for_cum_age_hc_use_depo_1[cond2t3+1] <- df$age_at_med_start_updated.3[cond2t3+1]

df$cumAgeHCUseDepoDiff <- ifelse(df$during_hc_use_depo == 0 & df$during_hc_use_depo_next > 0,df$for_cum_age_hc_use_depo_2 - df$for_cum_age_hc_use_depo_1,
                                ifelse(df$during_hc_use_depo > 0 & df$during_hc_use_depo_next == 0,df$for_cum_age_hc_use_depo_2 - df$for_cum_age_hc_use_depo_1,
                                ifelse(df$during_hc_use_depo > 0 & df$during_hc_use_depo_next > 0,df$for_cum_age_hc_use_depo_2 - df$for_cum_age_hc_use_depo_1,
                                ifelse(df$during_hc_use_depo == 0 & df$during_hc_use_depo_next == 0,0,NA))))

# Added 4/8/2022:
df$for_cum_age_hc_use_depo_1 <- ifelse(df$cd4_or_vl_12month_gap == 1,NA,df$for_cum_age_hc_use_depo_1)
df$for_cum_age_hc_use_depo_2 <- ifelse(df$cd4_or_vl_12month_gap == 1,NA,df$for_cum_age_hc_use_depo_2)
df$for_cum_age_hc_use_depo_2 <- ifelse(!is.na(df$cd4_or_vl_12month_gap_next) & df$cd4_or_vl_12month_gap == 0 &
                                       df$cd4_or_vl_12month_gap_next == 1, df$age_next, df$for_cum_age_hc_use_depo_2)
df$cumAgeHCUseDepoDiff <- ifelse(df$cd4_or_vl_12month_gap == 1,0,df$cumAgeHCUseDepoDiff)
df$cumAgeHCUseDepoDiff <- ifelse(!is.na(df$cd4_or_vl_12month_gap_next) & df$cd4_or_vl_12month_gap == 0 &
                                !is.na(df$cd4_or_vl_12month_gap) & df$cd4_or_vl_12month_gap_next == 1,
                                df$for_cum_age_hc_use_depo_2 - df$for_cum_age_hc_use_depo_1,df$cumAgeHCUseDepoDiff)
df$cumAgeHCUseDepoDiff[which(is.na(df$cumAgeHCUseDepoDiff))] <- 0
##################################

df$cumYrsHCUseDepo <- NA
split(df$cumYrsHCUseDepo, df$cfar_pid) <- lapply(split(df$cumAgeHCUseDepoDiff, df$cfar_pid), FUN=function(y){
                                             cumsum(y)
                                          })


df$cumYrsHCUseDepoMax <- NA
split(df$cumYrsHCUseDepoMax, df$cfar_pid) <- lapply(split(df$cumYrsHCUseDepo, df$cfar_pid), FUN=function(y){
                                               max(y, na.rm=TRUE) })
df$hc_use_depo <- as.numeric(df$cumYrsHCUseDepoMax > 0)
df$hc_use_depo.factor <- factor(df$hc_use_depo, levels=c(0,1), labels=c('No Depo Use','Depo Use'))

# And creating an indicator (0/1) for on/off HC (time-varying):
df$hc_use_depo_indicator <- ifelse(df$during_hc_use_depo > 0,1,0)
df$hc_use_depo_indicator.factor <- factor(df$hc_use_depo_indicator, levels=c(0,1), labels=c('No','Yes'))
label(df$hc_use_depo_indicator.factor) <- 'Depo HC use indicator (time-varying)'

# ---------------------------------- #
#    Multinomial HC use variable     #
# ---------------------------------- #
df$hc_use_mult_indicator <- ifelse(df$during_hc_use_oral == 0 & df$during_hc_use_depo == 0,0,
                                   ifelse(df$during_hc_use_oral > 0 & df$during_hc_use_depo == 0,1,
                                   ifelse(df$during_hc_use_oral == 0 & df$during_hc_use_depo > 0,2,
                                   ifelse(df$during_hc_use_oral > 0 & df$during_hc_use_depo > 0,2,NA))))
df$hc_use_mult_indicator.factor <- factor(df$hc_use_mult,
                                          levels=c(0,1,2),
                                          labels=c('No HC use',
                                                   'Oral HC use',
                                                   'Depo use'))
label(df$hc_use_mult_indicator.factor) <- 'Hormonal contraceptive use (multi-level)'


# ------------------------------------------------------------------ #
## Matching in non-varying data from cvdDx data and addDx data:
#df$nafld_outcome_partial_overall <- cvdDx[match(df$cfar_pid, cvdDx$cfar_pid, nomatch=NA),'nafld_outcome_partial_overall']
#df$nafld_outcome_partial_age_overall <- cvdDx[match(df$cfar_pid, cvdDx$cfar_pid, nomatch=NA),'nafld_outcome_partial_age_overall']
#df$metabolic_outcome_partial_overall <- addDx[match(df$cfar_pid, addDx$cfar_pid, nomatch=NA),'metabolic_outcome_partial_overall']
#df$metabolic_outcome_partial_age_overall <- addDx[match(df$cfar_pid, addDx$cfar_pid, nomatch=NA),'metabolic_outcome_partial_age_overall']

## Combining NAFLD and partial metabolic outcome for full metabolic outcome:
#df$metabolic_outcome_overall <- apply(df[,c('metabolic_outcome_partial_overall','nafld_outcome_partial_overall')], 1, FUN=function(y){
#                                   if(any(!is.na(y))){
#                                      tmp <- as.numeric(any(y[!is.na(y)] > 0))
#                                   } else {
#                                      tmp <- 0
#                                   }
#                                   return(tmp)
#                                })
#df$metabolic_outcome_age_overall <- apply(df[,c('metabolic_outcome_partial_age_overall','nafld_outcome_partial_age_overall')], 1, FUN=function(y){
#                                       if(any(!is.na(y))){
#                                          tmp <- min(y, na.rm=TRUE)
#                                       } else {
#                                          tmp <- NA
#                                       }
#                                       return(tmp)
#                                    })

# In order to determine cumulative HC use, I need to
# 1)  exclude time during pregnancy (between age_at_conception and age_at_delivery)
# 2)  censor if any gap in visit exists that is > 1 year for oral HCs and > 3 months for DMPA
# --------------------------- #
#    Pregnancy indicator      #
# --------------------------- #
conceptionVars <- grep('conception',names(df),value=TRUE)
deliveryVars <- grep('delivery',names(df),value=TRUE)
maxNum <- max(as.numeric(sapply(strsplit(conceptionVars, '[.]'),'[[',2)))

## Still need to work on this -- to create indicator that is 1 when pregnant for cumulative HC use
## Also, just noticed there are ages at conception that are after age at last visit:
jk <- apply(df, 1, FUN=function(y){
   indCum <- NULL
   for(j in 1:maxNum){
     nme1 <- paste0('age_at_conception.',j)
     nme2 <- paste0('age_at_delivery.',j)
     ind <- as.numeric(!is.na(y[[nme1]]) &
                       y[['age']] >= y[[nme1]] &
                       y[['age']] <= y[[nme2]])
     indCum <- c(indCum, ind)
   }
   indCum <- max(indCum)
   return(indCum) })

# ---------------------------------- #
#        Maximum follow-up time      #
# ---------------------------------- #
df$followUpTimeMax <- NA
split(df$followUpTimeMax, df$cfar_pid) <- lapply(split(df[,c('cfar_pid','age')], df$cfar_pid), FUN=function(y){
                                             tmp <- as.numeric(max(y[['age']]) - min(y[['age']]))
                                             return(tmp) })


# ---------------------------------- #
#        Creating factors            #
# ---------------------------------- #
vars2chkFactor <- paste0(vars2chk, '.factor')
df[,vars2chkFactor] <- map_dfc(.x = df[,vars2chk],
                               .f = function(y){
                                  factor(y, levels=c(0,1), labels=c('No','Yes'))
                               })
if(FALSE){
# ---------------------------------- #
#     Adding in time-varying CD4     #
# ---------------------------------- #
df$original_data <- 1
cd4D$age_at_first_visit <- df[match(cd4D$cfar_pid, df$cfar_pid, nomatch=NA),'age_at_first_visit']
cd4D_sub <- cd4D[cd4D$age_at_result_date >= cd4D$age_at_first_visit,c('cfar_pid','age_at_result_date','result_numeric','result')]
names(cd4D_sub) <- c('cfar_pid','age_at_cd4_result','cd4_numeric','cd4')

df2 <- merge(df, cd4D_sub, by.x=c('cfar_pid','age'), by.y = c('cfar_pid','age_at_cd4_result'),all=TRUE)
condNA <- is.na(df2$original_data)
df2$age_at_cd4_result <- NA
df2$age_at_cd4_result[condNA] <- df2$age[condNA]

# 994413A has no CD4 counts recorded in CD4 data:
df2$cd4_numeric <- ifelse(is.na(df2$cd4_numeric) & df2$month == 1 & !is.na(df2$cd4_at_first_visit),
                          df2$cd4_at_first_visit,df2$cd4_numeric)
df2$cd4 <- ifelse(is.na(df2$cd4) & df2$month == 1 & !is.na(df2$cd4_at_first_visit),
                          df2$cd4_at_first_visit,df2$cd4)
df2$age_at_cd4_result <- ifelse(is.na(df2$age_at_cd4_result) & df2$month == 1 & !is.na(df2$age_cd4_at_first_visit),
                                df2$age_cd4_at_first_visit,df2$age_at_cd4_result)
df2$cd4_numeric_updated <- unsplit(lapply(split(df2$cd4_numeric, df2$cfar_pid), FUN=function(y){
                             L <- !is.na(y);y[c(NA,which(L))[cumsum(L)+1]]}),
                                  df2$cfar_pid)
df2$cd4_updated <- unsplit(lapply(split(df2$cd4, df2$cfar_pid), FUN=function(y){
                             L <- !is.na(y);y[c(NA,which(L))[cumsum(L)+1]]}),
                                  df2$cfar_pid)

df2$age_at_cd4_result_updated <- unsplit(lapply(split(df2$age_at_cd4_result, df2$cfar_pid), FUN=function(y){
                                    L <- !is.na(y)
                                    y[c(NA,which(L))[cumsum(L)+1]]}),
                                    df2$cfar_pid)

df <- df2[!is.na(df2$original_data),]
df$cd4_numeric <- df$cd4 <- df$age_at_cd4_result <- NULL
df$cd4_numeric <- df$cd4_numeric_updated
df$cd4 <- df$cd4_updated
df$age_at_cd4_result <- df$age_at_cd4_result
df$cd4_numeric_updated <- df$cd4_updated <- df$age_at_cd4_result_updated <- NULL


# Adjusting a few CD4 counts based on closest but after the age listed for the record but within 1 month of it:
df$cd4_age_diff <- 365.25*(df$age - df$age_at_cd4_result)

df$age_at_cd4_result_next <- NA
split(df$age_at_cd4_result_next, df$cfar_pid) <- lapply(split(df$age_at_cd4_result, df$cfar_pid), FUN=function(y){
                                                   if(length(y) > 1){
                                                      tmp <- c(y[2:length(y)], NA)
                                                   } else {
                                                      tmp <- NA
                                                   }
                                                   return(tmp) })
df$cd4_age_diff_next <- 365.5*(df$age_at_cd4_result_next - df$age)

condInd <- which(df$cd4_age_diff > df$cd4_age_diff_next & df$cd4_age_diff_next > 0 & df$cd4_age_diff_next <= 28.0)
df$cd4_numeric[condInd] <- df$cd4_numeric[(condInd+1)]
df$cd4[condInd] <- df$cd4[(condInd+1)]
df$age_at_cd4_result[condInd] <- df$age_at_cd4_result[(condInd+1)]
df$age_at_cd4_result_next[condInd] <- df$age_at_cd4_result_next[(condInd+1)]

}

if(FALSE){
# ---------------------------------- #
#  Adding in time-varying HIV-1 RNA  #
# ---------------------------------- #
condVL <- labsO$testname == 'HIV-1 RNA'
rnaD <- labsO[condVL,]

rnaD$age_at_first_visit <- df[match(rnaD$cfar_pid, df$cfar_pid, nomatch=NA),'age_at_first_visit']
rnaD_sub <- rnaD[rnaD$age_at_result_date >= rnaD$age_at_first_visit,c('cfar_pid','age_at_result_date','result_numeric','result')]
names(rnaD_sub) <- c('cfar_pid','age_at_vl_result','vl_numeric','vl')

df2 <- merge(df, rnaD_sub, by.x=c('cfar_pid','age'), by.y = c('cfar_pid','age_at_vl_result'),all=TRUE)
condNA <- is.na(df2$original_data)
df2$age_at_vl_result <- NA
df2$age_at_vl_result[condNA] <- df2$age[condNA]

df2$vl_numeric <- ifelse(is.na(df2$vl_numeric) & df2$month == 1 & !is.na(df2$vl_at_first_visit),
                          df2$vl_at_first_visit,df2$vl_numeric)
df2$vl <- ifelse(is.na(df2$vl) & df2$month == 1 & !is.na(df2$vl_at_first_visit),
                          df2$vl_at_first_visit,df2$vl)
df2$age_at_vl_result <- ifelse(is.na(df2$age_at_vl_result) & df2$month == 1 & !is.na(df2$age_vl_at_first_visit),
                                df2$age_vl_at_first_visit,df2$age_at_vl_result)
df2$vl_numeric_updated <- unsplit(lapply(split(df2$vl_numeric, df2$cfar_pid), FUN=function(y){
                             L <- !is.na(y);y[c(NA,which(L))[cumsum(L)+1]]}),
                                  df2$cfar_pid)
df2$vl_updated <- unsplit(lapply(split(df2$vl, df2$cfar_pid), FUN=function(y){
                             L <- !is.na(y);y[c(NA,which(L))[cumsum(L)+1]]}),
                                  df2$cfar_pid)

df2$age_at_vl_result_updated <- unsplit(lapply(split(df2$age_at_vl_result, df2$cfar_pid), FUN=function(y){
                                    L <- !is.na(y)
                                    y[c(NA,which(L))[cumsum(L)+1]]}),
                                    df2$cfar_pid)

df <- df2[!is.na(df2$original_data),]
df$vl_numeric <- df$vl <- df$age_at_vl_result <- NULL
df$vl_numeric <- df$vl_numeric_updated
df$vl <- df$vl_updated
df$age_at_vl_result <- df$age_at_vl_result
df$vl_numeric_updated <- df$vl_updated <- df$age_at_vl_result_updated <- NULL

# Adjusting a few viral loads based on closest but after the age listed for the record but within 1 month of it:
df$vl_age_diff <- 365.25*(df$age - df$age_at_vl_result)

df$age_at_vl_result_next <- NA
split(df$age_at_vl_result_next, df$cfar_pid) <- lapply(split(df$age_at_vl_result, df$cfar_pid), FUN=function(y){
                                                   if(length(y) > 1){
                                                      tmp <- c(y[2:length(y)], NA)
                                                   } else {
                                                      tmp <- NA
                                                   }
                                                   return(tmp) })
df$vl_age_diff_next <- 365.5*(df$age_at_vl_result_next - df$age)

condInd <- which(df$vl_age_diff > df$vl_age_diff_next & df$vl_age_diff_next > 0 & df$vl_age_diff_next <= 28.0)
df$vl_numeric[condInd] <- df$vl_numeric[(condInd+1)]
df$log10_vl_numeric <- log10(df$vl_numeric)
df$vl[condInd] <- df$vl[(condInd+1)]
df$age_at_vl_result[condInd] <- df$age_at_vl_result[(condInd+1)]
df$age_at_vl_result_next[condInd] <- df$age_at_vl_result_next[(condInd+1)]

}



# --------------------------------------- #
#   Adding in time-varying ART exposure   #
# --------------------------------------- #
art$age_at_regstart_updatedOriginal <- art$age_at_regstart_updated
artVars2Keep <- Cs(cfar_pid,
                   regimen,
                   art_number,
                   max_art_number,
                   age_at_regstart_updated,
                   age_at_regstart_updatedOriginal,
                   age_at_regstop,
                   days_on_regimen,
                   art_use,
                   art_use.factor,
                   art_at_baseline,
                   art_at_baseline.factor)
art_sub <- art[,c(artVars2Keep)]

df2 <- merge(df, art_sub, by.x=c('cfar_pid','age'), by.y = c('cfar_pid','age_at_regstart_updated'),all=TRUE)

df2$month <- unsplit(lapply(split(df2$month, df2$cfar_pid), FUN=function(y){
                             L <- !is.na(y);y[c(NA,which(L))[cumsum(L)+1]]}),
                                  df2$cfar_pid)

df2$age_at_regstart_updatedOriginal <- unsplit(lapply(split(df2$age_at_regstart_updatedOriginal, df2$cfar_pid), FUN=function(y){
                                          L <- !is.na(y); y[c(NA,which(L))[cumsum(L)+1]]}),
                                               df2$cfar_pid)
df2$age_at_regstop <- unsplit(lapply(split(df2$age_at_regstop, df2$cfar_pid), FUN=function(y){
                                          L <- !is.na(y); y[c(NA,which(L))[cumsum(L)+1]]}),
                                               df2$cfar_pid)
df2$during_pregnancy <- unsplit(lapply(split(df2$during_pregnancy, df2$cfar_pid), FUN=function(y){
                                          L <- !is.na(y); y[c(NA,which(L))[cumsum(L)+1]]}),
                                               df2$cfar_pid)
df2$during_pregnancy_next <- unsplit(lapply(split(df2$during_pregnancy_next, df2$cfar_pid), FUN=function(y){
                                          L <- !is.na(y); y[c(NA,which(L))[cumsum(L)+1]]}),
                                               df2$cfar_pid)

df2$age_at_conception.1 <- unsplit(lapply(split(df2$age_at_conception.1, df2$cfar_pid), FUN=function(y){
                                          L <- !is.na(y); y[c(NA,which(L))[cumsum(L)+1]]}),
                                               df2$cfar_pid)
df2$age_at_conception.2 <- unsplit(lapply(split(df2$age_at_conception.2, df2$cfar_pid), FUN=function(y){
                                          L <- !is.na(y); y[c(NA,which(L))[cumsum(L)+1]]}),
                                               df2$cfar_pid)
df2$age_at_conception.3 <- unsplit(lapply(split(df2$age_at_conception.3, df2$cfar_pid), FUN=function(y){
                                          L <- !is.na(y); y[c(NA,which(L))[cumsum(L)+1]]}),
                                               df2$cfar_pid)
df2$age_at_conception.4 <- unsplit(lapply(split(df2$age_at_conception.4, df2$cfar_pid), FUN=function(y){
                                          L <- !is.na(y); y[c(NA,which(L))[cumsum(L)+1]]}),
                                               df2$cfar_pid)

df2$age_at_delivery.1 <- unsplit(lapply(split(df2$age_at_delivery.1, df2$cfar_pid), FUN=function(y){
                                          L <- !is.na(y); y[c(NA,which(L))[cumsum(L)+1]]}),
                                               df2$cfar_pid)
df2$age_at_delivery.2 <- unsplit(lapply(split(df2$age_at_delivery.2, df2$cfar_pid), FUN=function(y){
                                          L <- !is.na(y); y[c(NA,which(L))[cumsum(L)+1]]}),
                                               df2$cfar_pid)
df2$age_at_delivery.3 <- unsplit(lapply(split(df2$age_at_delivery.3, df2$cfar_pid), FUN=function(y){
                                          L <- !is.na(y); y[c(NA,which(L))[cumsum(L)+1]]}),
                                               df2$cfar_pid)
df2$age_at_delivery.4 <- unsplit(lapply(split(df2$age_at_delivery.4, df2$cfar_pid), FUN=function(y){
                                          L <- !is.na(y); y[c(NA,which(L))[cumsum(L)+1]]}),
                                               df2$cfar_pid)
df2$art_use <- unsplit(lapply(split(df2$art_use, df2$cfar_pid), FUN=function(y){
                                          L <- !is.na(y); y[c(NA,which(L))[cumsum(L)+1]]}),
                                               df2$cfar_pid)
df2$art_use.factor <- unsplit(lapply(split(df2$art_use.factor, df2$cfar_pid), FUN=function(y){
                                          L <- !is.na(y); y[c(NA,which(L))[cumsum(L)+1]]}),
                                               df2$cfar_pid)
                                               

# Added 3/26/2022:
df2$cd4_or_vl_12month_gap_updated <- unsplit(lapply(split(df2$cd4_or_vl_12month_gap, df2$cfar_pid), FUN=function(y){
                                         L <- !is.na(y); y[c(NA,which(L))[cumsum(L)+1]]}),
                                              df2$cfar_pid)
df2$cd4_or_vl_12month_gap_original <- df2$cd4_or_vl_12month_gap
df2$cd4_or_vl_12month_gap <- df2$cd4_or_vl_12month_gap_updated
# There are 13 IDs only in ART and not in the df data frame so will set their cd4_or_vl_12month_gap value to 0
df2$cd4_or_vl_12month_gap[which(is.na(df2$cd4_or_vl_12month_gap))] <- 0
###########################################

df2$during_art <- ifelse(!is.na(df2$age_at_regstart_updatedOriginal) &
                         df2$age >= df2$age_at_regstart_updatedOriginal & df2$age <= df2$age_at_regstop,1,0)
                         

# Added 3/26/2022:
# Setting those during_art indicators that are 1 to 0 if during a lab gap
df2$during_art <- ifelse(df2$during_art == 1 & df2$cd4_or_vl_12month_gap == 1,0,df2$during_art)
########################################
df2$cfar_pid <- as.character(df2$cfar_pid)
df2$during_art_next <- NA
split(df2$during_art_next, df2$cfar_pid) <- lapply(split(df2$during_art, df2$cfar_pid), FUN=function(y){
                                                 c(y[-1],NA)
                                              })
df2$age_art_next <- NA
split(df2$age_art_next, df2$cfar_pid) <- lapply(split(df2$age, df2$cfar_pid), FUN=function(y){
                                                 c(y[-1],NA)
                                              })


df2$for_cum_art_1 <- ifelse(!is.na(df2$age_at_regstart_updatedOriginal) & df2$age_at_regstart_updatedOriginal > df2$age,
                            df2$age_at_regstart_updated,df2$age)


df2$for_cum_art_1 <- ifelse(df2$during_pregnancy == 0, df2$for_cum_art_1,
                              ifelse(!is.na(df2$during_art) & df2$during_art == 0,-1000,
                              ifelse(is.na(df2$age_at_regstart_updatedOriginal),-1000,

                                    ifelse(df2$during_pregnancy > 0 & df2$during_pregnancy == df2$during_pregnancy_next,-1000,

                                    ifelse(df2$during_pregnancy == 1 & df2$during_pregnancy_next == 1 & df2$during_art == 1 &
                                           df2$age >= df2$age_at_conception.1,-991,
                                    ifelse(df2$during_pregnancy == 1 & df2$during_pregnancy_next == 0 & df2$during_art == 1 &
                                           df2$for_cum_art_1 < df2$age_at_delivery.1,df2$age_at_delivery.1,

                                    ifelse(df2$during_pregnancy == 2 & df2$during_pregnancy_next == 2 & df2$during_art == 1 &
                                           df2$age >= df2$age_at_conception.2 & round(df2$age,4) <= round(df2$age_at_delivery.2,4),-992,
                                    ifelse(df2$during_pregnancy == 2 & df2$during_pregnancy_next == 0 & df2$during_art == 1 &
                                           df2$for_cum_art_1 < df2$age_at_delivery.2,df2$age_at_delivery.2,
                                    ifelse(df2$during_pregnancy == 2 & df2$during_pregnancy_next == 0 & df2$during_art == 1 &
                                           df2$for_cum_art_1 == df2$age_at_delivery.2,-1000,


                                    ifelse(df2$during_pregnancy == 3 & df2$during_pregnancy_next == 3 & df2$during_art == 1 &
                                           df2$age >= df2$age_at_conception.3 & df2$age < df2$age_at_delivery.3,-993,
                                    ifelse(df2$during_pregnancy == 3 & df2$during_pregnancy_next == 0 & df2$during_art == 1 &
                                           df2$for_cum_art_1 < df2$age_at_delivery.3,df2$age_at_delivery.3,

                                    ifelse(df2$during_pregnancy == 4 & df2$during_pregnancy_next == 4 & df2$during_art == 1 &
                                           df2$age >= df2$age_at_conception.4 & df2$age < df2$age_at_delivery.4,-994,
                                    ifelse(df2$during_pregnancy == 4 & df2$during_pregnancy_next == 0 & df2$during_art == 1 &
                                           df2$for_cum_art_1 < df2$age_at_delivery.4,df2$age_at_delivery.4,
                                           df2$for_cum_art_1)))))))))))))




df2$for_cum_art_2 <- ifelse(!is.na(df2$age_at_regstop) & df2$age_at_regstop < df2$age_art_next,
                            df2$age_at_regstop,df2$age_art_next)
df2$for_cum_art_2 <- ifelse(is.na(df2$for_cum_art_2) & df2$during_art == 1 & is.na(df2$during_art_next) &
                            df2$age >= df2$age_at_regstart_updatedOriginal & df2$age < df2$age_at_regstop,
                            df2$age_at_regstop,df2$for_cum_art_2)
df2$for_cum_art_2 <- ifelse(df2$for_cum_art_1 < 0, -1000, df2$for_cum_art_2)
df2$for_cum_art_2 <- ifelse(df2$for_cum_art_2 < df2$for_cum_art_1 & df2$during_art == 1,df2$for_cum_art_1,df2$for_cum_art_2)


df2$cumARTDays <- ifelse(df2$during_art == 1 & df2$for_cum_art_1 >= 0, 365.25*(df2$for_cum_art_2 - df2$for_cum_art_1),0)
df2$id_month <- paste(df2$cfar_pid,df2$month,sep='.')
df2$cumARTDaysTot <- NA
split(df2$cumARTDaysTot, df2$id_month) <- lapply(split(df2$cumARTDays, df2$id_month), FUN=function(y){
                                             sum(y, na.rm=TRUE) })
df2$during_art_updated <- ifelse(df2$cumARTDaysTot >= 15.0 & df2$during_art == 1,1,0)

df2$art_useTot <- NA
split(df2$art_useTot, df2$id_month) <- lapply(split(df2[,c('art_use','cumARTDays','during_art_updated')], df2$id_month),
                                              FUN=function(y){
                                                 y_sub <- y[which(y[['during_art_updated']] == 1),]
                                                 if(nrow(y_sub) > 0 & length(unique(y[['art_use']])) == 1){
                                                    tmp <- y[['art_use']]
                                                 } else if(nrow(y_sub) > 0 & length(unique(y[['art_use']])) > 1){
                                                    which.row <- which(y[['cumARTDays']] == max(y[['cumARTDays']],na.rm=TRUE))
                                                    tmp <- y[which.row,'art_use']
                                                 } else {
                                                    tmp <- NA
                                                 }
                                                 return(tmp) })
# Adding 1 to each art_useTot so that 0 can be for those without ART exposure in a month:
df2$art_useTot <- df2$art_useTot + 1
df2$art_useTot <- ifelse(is.na(df2$art_useTot),0,df2$art_useTot)
df2$art_useTot.factor <- factor(df2$art_useTot, levels=c(0,1,2,3), labels=c('No ART','PI-use','INSTI-use','Other ART use'))

df <- df2[which(!is.na(df2$original_data)),]
df$art_at_baseline[which(is.na(df$art_at_baseline))] <- 0
df$art_at_baseline.factor <- factor(df$art_at_baseline,
                                    levels=c(0,1,2,3),
                                    labels=c('No ART',
                                             'PI-use',
                                             'INSTI-use',
                                             'Other ART use'))
df$during_art_updated_next <- NA
split(df$during_art_updated_next, df$cfar_pid) <- lapply(split(df$during_art_updated, df$cfar_pid), FUN=function(y){
                                                     c(y[-1],NA)
                                                  })

# Having to adjust a few stragglers at the start of regimens whose ART indicator was 0
df$during_art_updated <- ifelse(df$art_useTot > 0 & df$during_art_updated == 0,1,df$during_art_updated)

#tmp <- df2[which(df2$during_art_updated == 1),]
#jr <- tapply(tmp$art_use.factor, INDEX=tmp$id_month, FUN=function(y){length(unique(y[which(!is.na(y))]))})


df$metabolic_disorder_event_time_varying <- ifelse((!is.na(df$metabolic_disorder_event_overall_age) &
                                                    df$age >= df$metabolic_disorder_event_overall_age) |
                                             (!is.na(df$metabolic_disorder_event_overall_age) &
                                             df$age < df$metabolic_disorder_event_overall_age &
                                             !is.na(df$age_pregnancy_next) & df$age_pregnancy_next > df$metabolic_disorder_event_overall_age) ,1,
                                              ifelse((!is.na(df$metabolic_disorder_event_overall_age) & df$age < df$metabolic_disorder_event_overall_age) |
                                                     is.na(df$metabolic_disorder_event_overall_age),0,NA))
df$metabolic_disorder_event <- ifelse(!is.na(df$metabolic_disorder_event_overall_age) &
                                      !is.na(df$age_pregnancy_next) &
                                      df$age <= df$metabolic_disorder_event_overall_age &
                                      df$age_pregnancy_next > df$metabolic_disorder_event_overall_age,1,

                                      ifelse(!is.na(df$metabolic_disorder_event_overall_age) &
                                             df$age <= df$metabolic_disorder_event_overall_age &
                                             is.na(df$age_pregnancy_next),1,0))


df$cvd_event_time_varying <- ifelse((!is.na(df$cvd_event_overall_age) & df$age >= df$cvd_event_overall_age) |
                                    (!is.na(df$cvd_event_overall_age) & df$age < df$cvd_event_overall_age &
                                    !is.na(df$age_pregnancy_next) & df$age_pregnancy_next > df$cvd_event_overall_age) ,1,
                              ifelse((!is.na(df$cvd_event_overall_age) & df$age < df$cvd_event_overall_age) |
                                     is.na(df$cvd_event_overall_age),0,NA))

df$cvd_event <- ifelse(!is.na(df$cvd_event_overall_age) &
                       !is.na(df$age_pregnancy_next) &
                       df$age <= df$cvd_event_overall_age &
                       df$age_pregnancy_next > df$cvd_event_overall_age,1,
                       ifelse(!is.na(df$cvd_event_overall_age) &
                              df$age <= df$cvd_event_overall_age &
                              is.na(df$age_pregnancy_next),1,0))


df$thrombosis_event_time_varying <- ifelse((!is.na(df$thrombosis_event_overall_age) & df$age >= df$thrombosis_event_overall_age) |
                                    (!is.na(df$thrombosis_event_overall_age) & df$age < df$thrombosis_event_overall_age &
                                    !is.na(df$age_pregnancy_next) & df$age_pregnancy_next > df$thrombosis_event_overall_age) ,1,
                              ifelse((!is.na(df$thrombosis_event_overall_age) & df$age < df$thrombosis_event_overall_age) |
                                     is.na(df$thrombosis_event_overall_age),0,NA))

df$thrombosis_event <- ifelse(!is.na(df$thrombosis_event_overall_age) &
                       !is.na(df$age_pregnancy_next) &
                       df$age <= df$thrombosis_event_overall_age &
                       df$age_pregnancy_next > df$thrombosis_event_overall_age,1,
                       ifelse(!is.na(df$thrombosis_event_overall_age) &
                              df$age <= df$thrombosis_event_overall_age &
                              is.na(df$age_pregnancy_next),1,0))

df$cvd_or_thrombosis_event_time_varying <- ifelse((!is.na(df$cvd_or_thrombosis_event_overall_age) &
                                                   df$age >= df$cvd_or_thrombosis_event_overall_age) |
                                                  (!is.na(df$cvd_or_thrombosis_event_overall_age) &
                                                    df$age < df$cvd_or_thrombosis_event_overall_age &
                                                    !is.na(df$age_pregnancy_next) & df$age_pregnancy_next > df$cvd_or_thrombosis_event_overall_age) ,1,
                                             ifelse((!is.na(df$cvd_or_thrombosis_event_overall_age) &
                                                     df$age < df$cvd_or_thrombosis_event_overall_age) |
                                                     is.na(df$cvd_or_thrombosis_event_overall_age),0,NA))

df$cvd_or_thrombosis_event <- ifelse(!is.na(df$cvd_or_thrombosis_event_overall_age) &
                                     !is.na(df$age_pregnancy_next) &
                                     df$age <= df$cvd_or_thrombosis_event_overall_age &
                                     df$age_pregnancy_next > df$cvd_or_thrombosis_event_overall_age,1,
                                     ifelse(!is.na(df$cvd_or_thrombosis_event_overall_age) &
                                            df$age <= df$cvd_or_thrombosis_event_overall_age &
                                            is.na(df$age_pregnancy_next),1,0))

# Added 3/26/2022 (comment)
# One ID whose cd4_or_vl_12month_gap_stop_age exceeds the age_at_lastvisit but not the age_at_censor so will leave alone
############################################################

# -------------------------------- #
#        Loss to follow-up         #
# -------------------------------- #
df$ltfu <- ifelse(!is.na(df$age_at_death),0,
                  ifelse(is.na(df$age_at_death) & df$age_at_censor != df$age_at_lastvisit,0,
                  ifelse(is.na(df$age_at_death) & df$age_at_censor == df$age_at_lastvisit,1,NA)))
df$ltfu.factor <- factor(df$ltfu, levels=c(0,1), labels=c('Not LTFU','LTFU'))
label(df$ltfu.factor) <- 'Lost to follow-up'

# Adding in dichotomous hispanic for Table 1:
df$hispanic_dichot <- ifelse(df$hispanic %in% c('No','Unknown'),0,1)
df$hispanic_dichot.factor <- factor(df$hispanic_dichot, levels=c(0,1), labels=c('No/Unknown','Yes'))
label(df$hispanic_dichot.factor) <- 'Hispanic (dichotomized)'

# Any ART use during follow-up:
df$artUseAny <- NA
split(df$artUseAny, df$cfar_pid) <- lapply(split(df$art_useTot, df$cfar_pid), FUN=function(y){
                                       as.numeric(any(y > 0))
                                    })
df$artUseAny.factor <- factor(df$artUseAny, 
                              levels=c(0,1), 
                              labels=c('No','Yes'))
label(df$artUseAny.factor) <- 'Any ART use during follow-up'
