# Excluding subjects with prevalent:
# 1)  hysterectomy
# 2)  cervical cancer
# 3)  breast cancer
# 4)  ovarian cancer
# 5)  bilateral tubal ligation
condPrevHysterectomy <- !is.na(patients$age_at_hysterectomy) & patients$age_at_hysterectomy < patients$age_at_first_visit
condPrevCervicalCa <- !is.na(patients$age_at_cervicalca) & patients$age_at_cervicalca < patients$age_at_first_visit
condPrevBreastCa <- !is.na(patients$age_at_breastca) & patients$age_at_breastca < patients$age_at_first_visit
condPrevOvarianCa <- !is.na(patients$age_at_ovarianca) & patients$age_at_ovarianca < patients$age_at_first_visit
condPrevBTL <- !is.na(patients$age_at_btl) & patients$age_at_btl < patients$age_at_first_visit

patients <- patients[!condPrevHysterectomy & 
                     !condPrevCervicalCa & 
                     !condPrevBreastCa & 
                     !condPrevOvarianCa & 
                     !condPrevBTL,]

# Adding in age_at_censor:
patients$age_at_censor <- censorVar[match(patients$cfar_pid,
                                          censorVar$cfar_pid,
                                          nomatch=NA),
                                    'age_at_censor']

# Defining factors:
patients$race.factor <- factor(patients$race, levels=c('White',
                                                       'Black',
                                                       'Asian',
                                                       'Multiracial',
                                                       'Other',
                                                       'Unknown'))
patients$birthsex.factor <- factor(patients$birthsex, levels=c('Female'))
patients$hispanic.factor <- factor(patients$hispanic, levels=c('No','Yes','Unknown'))

patients$race_3lvl <- ifelse(patients$race.factor == 'White' &
                             patients$hispanic.factor == 'No',0,
                             ifelse(patients$race.factor == 'Black' &
                                    patients$hispanic.factor == 'No',1,2))
patients$race_3lvl.factor <- factor(patients$race_3lvl,
                                    levels=c(0,1,2),
                                    labels=c('Non-Hispanic White',
                                             'Non-Hispanic Black',
                                             'Other'))
patients$race_2lvl <- ifelse(!is.na(patients$race.factor) & 
                             patients$race.factor == 'White',0,
                             ifelse(!is.na(patients$race.factor),1,NA))
patients$race_2lvl.factor <- factor(patients$race_2lvl, levels=c(0,1), labels=c('White','non-White'))
label(patients$race_2lvl.factor) <- 'Race (dichotomized)'

# For those with BTL or hysterectomy after age_at_censor, need to remove the event:
patients$btl_after_censor <- as.numeric(!is.na(patients$age_at_btl) & patients$age_at_btl > patients$age_at_censor)
patients$hysterectomy_after_censor <- as.numeric(!is.na(patients$age_at_hysterectomy) & patients$age_at_hysterectomy > patients$age_at_censor)
patients$age_at_btl[which(!is.na(patients$age_at_btl) & patients$age_at_btl > patients$age_at_censor)] <- NA
patients$age_at_hysterectomy[which(!is.na(patients$age_at_hysterectomy) & patients$age_at_hysterectomy > patients$age_at_censor)] <- NA
