pregnancies$age_at_first_visit <- patients[match(pregnancies$cfar_pid, patients$cfar_pid, nomatch=NA),'age_at_first_visit']

# Testing -- removing records of pregnancies where the delivery is before age_at_first_visit:
condBFPreg <- !is.na(pregnancies$age_at_first_visit) & !is.na(pregnancies$age_at_delivery) & 
              pregnancies$age_at_first_visit > pregnancies$age_at_delivery
pregnancies_original <- pregnancies
pregnancies <- pregnancies[!condBFPreg,]
              
# Cleaning up the pregnancy data with the following edits:
# ID 995626K where missing age at delivery --> age at delivery = 34.4641
cond1 <- is.na(pregnancies$age_at_delivery) & pregnancies$cfar_pid == '995626K'
pregnancies$age_at_delivery[cond1] <- 34.4641

# ID 996565L where age_at_conception > age_at_delivery --> age_at_conception = 33.3053
cond2 <- pregnancies$cfar_pid == '996565L' & pregnancies$age_at_conception > pregnancies$age_at_delivery
pregnancies$age_at_conception[cond2] <- 33.3053

# ID 996705A where age_at_conception > 31.323 --> age_at_conception = 30.3238
cond3 <- pregnancies$cfar_pid == '996705A' & pregnancies$age_at_conception > 31.323
pregnancies$age_at_conception[cond3] <- 30.3238

# ID 998595G where missing age_at_delivery
# ID 999043A where missing age_at_delivery
# ID 999772I where missing age_at_conception
# ID 998485J where missing age_at_conception
# ID 997366L where missing age_at_conception
# ID 996821E where missing age_at_conception
# ID 995883H where missing age_at_conception
cond4a <- pregnancies$cfar_pid == '998595G' & is.na(pregnancies$age_at_delivery)
cond4b <- pregnancies$cfar_pid == '999043A' & is.na(pregnancies$age_at_delivery)
cond4c <- pregnancies$cfar_pid == '999772I' & is.na(pregnancies$age_at_conception)
cond4d <- pregnancies$cfar_pid == '998485J' & is.na(pregnancies$age_at_conception)
cond4e <- pregnancies$cfar_pid == '997366L' & is.na(pregnancies$age_at_conception)
cond4f <- pregnancies$cfar_pid == '996821E' & is.na(pregnancies$age_at_conception)
cond4g <- pregnancies$cfar_pid == '995883H' & is.na(pregnancies$age_at_conception)
pregnancies <- pregnancies[!cond4a & !cond4b & !cond4c & !cond4d & !cond4e & !cond4f & !cond4g,]

# ID 995626K where age_at_delivery == 34.4641 has prgoutcome = Live Birth
cond5 <- pregnancies$cfar_pid == '995626K' & pregnancies$age_at_delivery == 34.4641
pregnancies$prgoutcome[cond5] <- 'Live Birth'

# ID 999043A has prgoutcome = 'Moved prior to delivery'
cond6 <- pregnancies$cfar_pid == '999043A'
pregnancies$prgoutcome[cond6] <- 'Moved prior to outcome'

# For those with a missing age_at_delivery whose pregnancy outcome was 'miscarriage' or 'abortion' will code the age_at_delivery to be 22 weeks after
# the age_at_conception
condNA <- is.na(pregnancies$age_at_delivery) & is.na(pregnancies$wks_at_delivery)
pregnancies$age_at_delivery[condNA] <- pregnancies$age_at_conception[condNA] + 22/52 # For those with missing age at delivery and miscarriage or unknown
# Imputing for missing age_at_conception or age_at_delivery:
condNAConception <- is.na(pregnancies$age_at_conception)
pregnancies$age_at_conception[condNAConception] <- pregnancies$age_at_delivery[condNAConception] - pregnancies$wks_at_delivery[condNAConception]/52

condNADelivery <- is.na(pregnancies$age_at_delivery)
pregnancies$age_at_delivery[condNADelivery] <- pregnancies$age_at_conception[condNADelivery] + pregnancies$wks_at_delivery[condNADelivery]/52


# Creating a wide data set to merge into monthly data:
pregnancies <- pregnancies[order(pregnancies$cfar_pid, pregnancies$age_at_conception),]
pregnancies$pregnancy_number <- NA
split(pregnancies$pregnancy_number, pregnancies$cfar_pid) <- lapply(split(pregnancies$cfar_pid, pregnancies$cfar_pid), FUN=function(y){
                                                                seq(1,length(y))})

pregnancies <- pregnancies[order(pregnancies$cfar_pid,pregnancies$age_at_conception),]

pregnancies_wide <- reshape(pregnancies, v.names = c('age_at_conception',
                                                        'age_at_delivery',
                                                        'prgoutcome',
                                                        'wks_at_delivery'),
                            idvar = 'cfar_pid',
                            timevar = 'pregnancy_number',
                            direction = 'wide')



# Need to define a baseline age (either age_at_first_visit for those who were not pregnant at that age or age_at_delivery for those who were pregnant
# at their first visit
pregnancies$age_at_1_month <- pregnancies$age_at_first_visit + 28/365.25
pregnancies$pregnant_at_first_visit <- ifelse(pregnancies$age_at_conception <= pregnancies$age_at_first_visit &
                                              pregnancies$age_at_delivery > pregnancies$age_at_first_visit,1,0)
condNewBaselineAge <- pregnancies$pregnant_at_first_visit == 1
pregnancies$new_baseline_age <- pregnancies$age_at_first_visit
pregnancies$new_baseline_age[condNewBaselineAge] <- pregnancies$age_at_delivery[condNewBaselineAge]

# Adding this to patients data:
patients$new_baseline_age <- pregnancies[match(patients$cfar_pid, pregnancies$cfar_pid, nomatch=NA),'new_baseline_age']
condNAAge <- is.na(patients$new_baseline_age)
patients$new_baseline_age[condNAAge] <- patients$age_at_first_visit[condNAAge]
